@@ -18,49 +18,49 @@
  */
 package org.apache.brooklyn.core.mgmt;
 
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
-import java.util.concurrent.Callable;
-
-import javax.annotation.Nullable;
-
-import org.apache.brooklyn.api.catalog.CatalogItem;
-import org.apache.brooklyn.api.entity.Application;
-import org.apache.brooklyn.api.entity.Entity;
-import org.apache.brooklyn.api.entity.EntitySpec;
-import org.apache.brooklyn.api.internal.AbstractBrooklynObjectSpec;
-import org.apache.brooklyn.api.mgmt.ManagementContext;
-import org.apache.brooklyn.api.mgmt.Task;
-import org.apache.brooklyn.config.ConfigKey;
-import org.apache.brooklyn.core.catalog.internal.BasicBrooklynCatalog;
-import org.apache.brooklyn.core.config.BasicConfigInheritance;
-import org.apache.brooklyn.core.config.ConfigKeys;
-import org.apache.brooklyn.core.effector.Effectors;
-import org.apache.brooklyn.core.entity.Entities;
-import org.apache.brooklyn.core.entity.EntityFunctions;
-import org.apache.brooklyn.core.entity.trait.Startable;
-import org.apache.brooklyn.core.mgmt.internal.EntityManagerInternal;
-import org.apache.brooklyn.core.typereg.RegisteredTypeLoadingContexts;
-import org.apache.brooklyn.entity.stock.BasicApplication;
-import org.apache.brooklyn.util.collections.MutableList;
-import org.apache.brooklyn.util.collections.MutableMap;
-import org.apache.brooklyn.util.core.task.TaskBuilder;
-import org.apache.brooklyn.util.core.task.Tasks;
-import org.apache.brooklyn.util.text.Strings;
-import org.apache.brooklyn.util.time.Duration;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
-
 import com.google.common.annotations.Beta;
 import com.google.common.base.Optional;
 import com.google.common.base.Predicates;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
-import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Iterables;
 import com.google.common.collect.Maps;
 import com.google.common.util.concurrent.Runnables;
+import org.apache.brooklyn.api.entity.Application;
+import org.apache.brooklyn.api.entity.Entity;
+import org.apache.brooklyn.api.entity.EntitySpec;
+import org.apache.brooklyn.api.mgmt.ManagementContext;
+import org.apache.brooklyn.api.mgmt.Task;
+import org.apache.brooklyn.config.ConfigKey;
+import org.apache.brooklyn.core.config.BasicConfigInheritance;
+import org.apache.brooklyn.core.config.ConfigKeys;
+import org.apache.brooklyn.core.effector.Effectors;
+import org.apache.brooklyn.core.entity.BrooklynConfigKeys;
+import org.apache.brooklyn.core.entity.Entities;
+import org.apache.brooklyn.core.entity.EntityFunctions;
+import org.apache.brooklyn.core.entity.trait.Startable;
+import org.apache.brooklyn.core.mgmt.internal.EntityManagerInternal;
+import org.apache.brooklyn.core.resolve.jackson.BeanWithTypeUtils;
+import org.apache.brooklyn.core.typereg.RegisteredTypeLoadingContexts;
+import org.apache.brooklyn.entity.stock.BasicApplication;
+import org.apache.brooklyn.util.collections.MutableList;
+import org.apache.brooklyn.util.collections.MutableMap;
+import org.apache.brooklyn.util.collections.MutableSet;
+import org.apache.brooklyn.util.core.flags.TypeCoercions;
+import org.apache.brooklyn.util.core.task.TaskBuilder;
+import org.apache.brooklyn.util.core.task.Tasks;
+import org.apache.brooklyn.util.exceptions.Exceptions;
+import org.apache.brooklyn.util.text.Strings;
+import org.apache.brooklyn.util.time.Duration;
+import org.apache.brooklyn.util.yaml.Yamls;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import javax.annotation.Nullable;
+import java.util.*;
+import java.util.concurrent.Callable;
+import java.util.function.Function;
+import java.util.stream.Collectors;
 
 /** Utility methods for working with entities and applications */
 public class EntityManagementUtils {
@@ -81,6 +81,11 @@ public class EntityManagementUtils {
             .runtimeInheritance(BasicConfigInheritance.NEVER_INHERITED)
             .build();
 
+    static final Set<String> ALLOWABLE_COLLAPSING_KEYS = MutableList.of(WRAPPER_APP_MARKER, BrooklynConfigKeys.TEMPLATE_ID).stream().map(ConfigKey::getName).collect(Collectors.toSet());
+
+    public static final boolean DIFFERENT_NAME_BLOCKS_UNWRAPPING = true;
+    public static final boolean DIFFERENT_CONFIG_BLOCKS_UNWRAPPING = true;
+
     /** creates an application from the given app spec, managed by the given management context */
     public static <T extends Application> T createUnstarted(ManagementContext mgmt, EntitySpec<T> spec) {
         return createUnstarted(mgmt, spec, Optional.absent());
@@ -91,8 +96,11 @@ public class EntityManagementUtils {
      */
     @Beta
     public static <T extends Application> T createUnstarted(ManagementContext mgmt, EntitySpec<T> spec, Optional<String> entityId) {
-        T app = ((EntityManagerInternal)mgmt.getEntityManager()).createEntity(spec, entityId);
-        return app;
+        return mgmt.getServerExecutionContext().get(Tasks.<T>builder().dynamic(false)
+            .displayName("Creating entity "+
+                (Strings.isNonBlank(spec.getDisplayName()) ? spec.getDisplayName() : spec.getType().getName()) )
+            .body(() -> ((EntityManagerInternal)mgmt.getEntityManager()).createEntity(spec, entityId))
+            .build() );
     }
 
     /** as {@link #createUnstarted(ManagementContext, EntitySpec)} but for a string plan (e.g. camp yaml) */
@@ -101,21 +109,12 @@ public class EntityManagementUtils {
         return createUnstarted(mgmt, spec);
     }
     
+    public static EntitySpec<? extends Application> createEntitySpecForApplication(ManagementContext mgmt, String plan) {
+        return createEntitySpecForApplication(mgmt, null, plan);
+    }
     @SuppressWarnings("unchecked")
-    public static EntitySpec<? extends Application> createEntitySpecForApplication(ManagementContext mgmt, final String plan) {
-        return mgmt.getTypeRegistry().createSpecFromPlan(null, plan, RegisteredTypeLoadingContexts.spec(Application.class), EntitySpec.class);
-    }
-
-    @Deprecated /** @deprecated since 0.9.0; use {@link BrooklynTypeRegistry#createSpec(RegisteredType, org.apache.brooklyn.api.typereg.RegisteredTypeConstraint, Class)} */
-    // not used in Brooklyn
-    public static <T,SpecT extends AbstractBrooklynObjectSpec<? extends T, SpecT>> SpecT createCatalogSpec(ManagementContext mgmt, CatalogItem<T, SpecT> item) {
-        return createCatalogSpec(mgmt, item, ImmutableSet.<String>of());
-    }
-
-    @Deprecated /** @deprecated since 0.9.0; use {@link BrooklynTypeRegistry#createSpec(RegisteredType, org.apache.brooklyn.api.typereg.RegisteredTypeConstraint, Class)} */
-    // not used in Brooklyn
-    public static <T,SpecT extends AbstractBrooklynObjectSpec<? extends T, SpecT>> SpecT createCatalogSpec(ManagementContext mgmt, final CatalogItem<T, SpecT> item, final Set<String> encounteredTypes) {
-        return BasicBrooklynCatalog.internalCreateSpecLegacy(mgmt, item, encounteredTypes, true);
+    public static EntitySpec<? extends Application> createEntitySpecForApplication(ManagementContext mgmt, String format, String plan) {
+        return mgmt.getTypeRegistry().createSpecFromPlan(format, plan, RegisteredTypeLoadingContexts.spec(Application.class), EntitySpec.class);
     }
 
     /** container for operation which creates something and which wants to return both
@@ -165,24 +164,11 @@ public class EntityManagementUtils {
     
     /** adds entities from the given yaml, under the given parent; but does not start them */
     public static List<Entity> addChildrenUnstarted(final Entity parent, String yaml) {
-        log.debug("Creating child of "+parent+" from yaml:\n{}", yaml);
+        log.debug("Creating under "+parent+" from yaml:\n{}", yaml);
 
         ManagementContext mgmt = parent.getApplication().getManagementContext();
 
-        EntitySpec<? extends Application> specA = createEntitySpecForApplication(mgmt, yaml);
-
-        // see whether we can promote children
-        List<EntitySpec<?>> specs = MutableList.of();
-        if (!canUnwrapEntity(specA)) {
-            // if not promoting, set a nice name if needed
-            if (Strings.isEmpty(specA.getDisplayName())) {
-                int size = specA.getChildren().size();
-                String childrenCountString = size+" "+(size!=1 ? "children" : "child");
-                specA.displayName("Dynamically added "+childrenCountString);
-            }
-        }
-        
-        specs.add(unwrapEntity(specA));
+        List<EntitySpec<?>> specs = getAddChildrenSpecs(mgmt, yaml);
 
         final List<Entity> children = MutableList.of();
         for (EntitySpec<?> spec: specs) {
@@ -193,6 +179,61 @@ public class EntityManagementUtils {
         return children;
     }
 
+    public static List<EntitySpec<?>> getAddChildrenSpecs(ManagementContext mgmt, String yamlMaybeList) {
+        // see whether there are multiple children, and can they be promoted
+
+        List<String> yamls = MutableList.of();
+        Object parse = Iterables.getOnlyElement(Yamls.parseAll(yamlMaybeList));
+        if (parse instanceof List) {
+            ((List) parse).forEach(li -> {
+                try {
+                    yamls.add(BeanWithTypeUtils.newYamlMapper(null, false, null, false).writeValueAsString(li));
+                } catch (Exception e) {
+                    throw Exceptions.propagateAnnotated("Invalid YAML for adding child", e);
+                }
+            });
+        } else {
+            yamls.add(yamlMaybeList); // not a list
+        }
+
+        List<EntitySpec<?>> result = MutableList.of();
+
+        yamls.forEach(yaml -> {
+            EntitySpec spec = null;
+            try {
+                spec = mgmt.getTypeRegistry().createSpecFromPlan(null, yaml, null, EntitySpec.class);
+            } catch (Exception e) {
+                Exceptions.propagateIfFatal(e);
+                try {
+                    Object yo = Iterables.getOnlyElement(Yamls.parseAll(yaml));
+                    // coercion does this at: org.apache.brooklyn.camp.brooklyn.spi.dsl.methods.BrooklynDslCommon.registerSpecCoercionAdapter
+                    // but is too forgiving -- will coerce an empty map; so we do an extra check
+                    spec = TypeCoercions.tryCoerce(yo, EntitySpec.class).orNull();
+                    if (spec.getType()==null && spec.getImplementation()==null) {
+                        if (log.isTraceEnabled()) log.trace("Failed converting entity spec YAML as YAML, transformer error will throw, but also created a bogus empty spec: "+spec+", from "+yo);
+                        spec = null;
+                    }
+                } catch (Exception e2) {
+                    log.debug("Failed converting entity spec YAML as YAML, transformer error will throw, but also encountered: "+e2);
+                }
+                if (spec==null) throw Exceptions.propagate(e);
+            }
+            if (!canUnwrapEntity(spec)) {
+                // if not promoting, set a nice name if needed
+                if (Strings.isEmpty(spec.getDisplayName())) {
+                    int size = spec.getChildren().size();
+                    if (size>0) {
+                        String childrenCountString = size==1 ? "child" : size + " children";
+                        spec.displayName("Dynamically added " + childrenCountString);
+                    }
+                }
+            }
+            result.add(spec);
+        });
+
+        return result;
+    }
+
     public static CreationResult<List<Entity>,List<String>> addChildrenStarting(final Entity parent, String yaml) {
         final List<Entity> children = addChildrenUnstarted(parent, yaml);
         String childrenCountString;
@@ -229,10 +270,14 @@ public class EntityManagementUtils {
         return CreationResult.of(children, task);
     }
     
+    public static EntitySpec<? extends Entity> unwrapEntity(EntitySpec<? extends Entity> wrapperApplication) {
+        return unwrapEntity(wrapperApplication, false);
+    }
+    
     /** Unwraps a single {@link Entity} if appropriate. See {@link #WRAPPER_APP_MARKER}.
      * Also see {@link #canUnwrapEntity(EntitySpec)} to test whether it will unwrap. */
-    public static EntitySpec<? extends Entity> unwrapEntity(EntitySpec<? extends Entity> wrapperApplication) {
-        if (!canUnwrapEntity(wrapperApplication)) {
+    public static EntitySpec<? extends Entity> unwrapEntity(EntitySpec<? extends Entity> wrapperApplication, boolean allowUnwrappingApplicationsWithoutWrapperAppMarker) {
+        if (!canUnwrapEntity(wrapperApplication, allowUnwrappingApplicationsWithoutWrapperAppMarker)) {
             return wrapperApplication;
         }
         EntitySpec<?> wrappedEntity = Iterables.getOnlyElement(wrapperApplication.getChildren());
@@ -254,7 +299,7 @@ public class EntityManagementUtils {
         EntitySpec<? extends Application> wrappedApplication = (EntitySpec<? extends Application>) unwrapEntity(wrapperApplication);
         return wrappedApplication;
     }
-    
+
     /** Modifies the child so it includes the inessential setup of its parent,
      * for use when unwrapping specific children, but a name or other item may have been set on the parent.
      * See {@link #WRAPPER_APP_MARKER}. */
@@ -270,8 +315,6 @@ public class EntityManagementUtils {
             wrappedChild.parametersAdd(wrapperParent.getParameters());
         }
 
-        wrappedChild.catalogItemIdAndSearchPath(wrapperParent.getCatalogItemId(), wrapperParent.getCatalogItemIdSearchPath());
-
         // NB: this clobber's child config wherever they conflict; might prefer to deeply merge maps etc
         // (or maybe even prevent the merge in these cases; 
         // not sure there is a compelling reason to have config on a pure-wrapper parent)
@@ -280,7 +323,25 @@ public class EntityManagementUtils {
                 Predicates.not(Predicates.<ConfigKey<?>>equalTo(EntityManagementUtils.WRAPPER_APP_MARKER)));
         wrappedChild.configure(configWithoutWrapperMarker);
         wrappedChild.configure(wrapperParent.getFlags());
-        
+
+        // add the search path to children when unwrapped, in preference to anything on the children
+        String preferredCatalogItemId, otherCatalogItemId;
+        if (wrapperParent.getCatalogItemId()!=null) {
+            preferredCatalogItemId = wrapperParent.getCatalogItemId();
+            otherCatalogItemId = wrappedChild.getCatalogItemId();
+            if (Objects.equals(otherCatalogItemId, preferredCatalogItemId)) {
+                otherCatalogItemId = null;
+            }
+        } else {
+            preferredCatalogItemId = wrappedChild.getCatalogItemId();
+            otherCatalogItemId = null;
+        }
+        MutableList<String> searchPath = MutableList.<String>of()
+                .appendAll(wrapperParent.getCatalogItemIdSearchPath())
+                .appendIfNotNull(otherCatalogItemId)
+                .appendAll(wrappedChild.getCatalogItemIdSearchPath());
+        wrappedChild.catalogItemIdAndSearchPath(preferredCatalogItemId, searchPath);
+
         // copying tags to all entities may be something the caller wants to control,
         // e.g. if we're adding multiple, the caller might not want to copy the parent
         // (the BrooklynTags.YAML_SPEC tag will include the parents source including siblings),
@@ -304,9 +365,9 @@ public class EntityManagementUtils {
         EntitySpec<?> childSpec = Iterables.getOnlyElement(wrapperApplication.getChildren());
         return (childSpec.getType()!=null && Application.class.isAssignableFrom(childSpec.getType()));
     }
-    /** @deprecated since 0.9.0 use {@link #canUnwrapApplication(EntitySpec)} */ @Deprecated
-    public static boolean canPromoteWrappedApplication(EntitySpec<? extends Application> app) {
-        return canUnwrapApplication(app);
+    
+    public static boolean canUnwrapEntity(EntitySpec<? extends Entity> spec) {
+        return canUnwrapEntity(spec, false);
     }
     
     /** Returns true if the spec is for a wrapper app with no important settings, wrapping a single child entity. 
@@ -316,26 +377,76 @@ public class EntityManagementUtils {
      * see {@link #canUnwrapApplication(EntitySpec)} if that is required).
      * <p>
      * Note callers will normally use one of {@link #unwrapEntity(EntitySpec)} or {@link #unwrapApplication(EntitySpec)}.
-     * 
-     * @see #WRAPPER_APP_MARKER for an overview */
-    public static boolean canUnwrapEntity(EntitySpec<? extends Entity> spec) {
-        return isWrapperApp(spec) && hasSingleChild(spec) &&
-            // these "brooklyn.*" items on the app rather than the child absolutely prevent unwrapping
-            // as their semantics could well be different whether they are on the parent or the child
-            spec.getEnrichers().isEmpty() &&
-            spec.getEnricherSpecs().isEmpty() &&
-            spec.getInitializers().isEmpty() &&
-            spec.getPolicies().isEmpty() &&
-            spec.getPolicySpecs().isEmpty() &&
-            // prevent merge only if a location is defined at both levels
-            ((spec.getLocations().isEmpty() && spec.getLocationSpecs().isEmpty()) || 
-                (Iterables.getOnlyElement(spec.getChildren()).getLocations().isEmpty()) && Iterables.getOnlyElement(spec.getChildren()).getLocationSpecs().isEmpty())
-            // parameters are collapsed on merge so don't need to be considered here
-            ;
+     *
+     * @see #WRAPPER_APP_MARKER  */
+    public static boolean canUnwrapEntity(EntitySpec<? extends Entity> spec, boolean allowUnwrappingApplicationsWithoutWrapperAppMarker) {
+        return ((allowUnwrappingApplicationsWithoutWrapperAppMarker && Application.class.isAssignableFrom(spec.getType()))
+                || isWrapperApp(spec)) &&
+            hasMergeableSingleChild(spec);
     }
-    /** @deprecated since 0.9.0 use {@link #canUnwrapEntity(EntitySpec)} */ @Deprecated
-    public static boolean canPromoteChildrenInWrappedApplication(EntitySpec<? extends Application> spec) {
-        return canUnwrapEntity(spec);
+
+    private static boolean hasMergeableSingleChild(EntitySpec<? extends Entity> spec) {
+        if (!hasSingleChild(spec)) return false;
+        // these "brooklyn.*" items on the app rather than the child absolutely prevent unwrapping
+        // as their semantics could well be different whether they are on the parent or the child
+        EntitySpec<? extends Entity> child = Iterables.getOnlyElement(spec.getChildren());
+        if (!allParentSubsetOfChild(spec, child, EntitySpec::getEnricherSpecs, EntitySpec::getInitializers, EntitySpec::getPolicySpecs)) return false;
+
+        // prevent merge if different names defined
+        if (DIFFERENT_NAME_BLOCKS_UNWRAPPING) {
+            if (Strings.isNonBlank(spec.getDisplayName()) && Strings.isNonBlank(child.getDisplayName()) && !spec.getDisplayName().equals(child.getDisplayName()))
+                return false;
+        }
+
+        if (DIFFERENT_CONFIG_BLOCKS_UNWRAPPING) {
+            // prevent merge if conflicting config (apart from selected expected-different keys); includes 'id'
+            Map<String, Object> configAndFlagValues = MutableMap.of();
+            for (Map.Entry<String, ?> entry : spec.getFlags().entrySet()) {
+                String kn = entry.getKey();
+                if (ALLOWABLE_COLLAPSING_KEYS.contains(kn)) continue;
+                configAndFlagValues.put(kn, entry.getValue());
+            }
+            for (Map.Entry<ConfigKey<?>, Object> entry : spec.getConfig().entrySet()) {
+                String kn = entry.getKey().getName();
+                if (ALLOWABLE_COLLAPSING_KEYS.contains(kn)) continue;
+                configAndFlagValues.put(kn, entry.getValue());
+            }
+
+            for (Map.Entry<String, ?> entry : child.getFlags().entrySet()) {
+                String kn = entry.getKey();
+                Object v = configAndFlagValues.get(kn);
+                if (v != null && !v.equals(entry.getValue())) return false;
+            }
+            for (Map.Entry<ConfigKey<?>, Object> entry : child.getConfig().entrySet()) {
+                String kn = entry.getKey().getName();
+                Object v = configAndFlagValues.get(kn);
+                if (v != null && !v.equals(entry.getValue())) return false;
+            }
+        }
+
+        // prevent merge if a location is defined at both levels
+        if (! ((spec.getLocations().isEmpty() && spec.getLocationSpecs().isEmpty()) ||
+                (Iterables.getOnlyElement(spec.getChildren()).getLocations().isEmpty()) && child.getLocationSpecs().isEmpty())) {
+            return false;
+        }
+
+        Set<String> parentParamsWithDefaults = spec.getParameters().stream().filter(p -> p.getConfigKey().getDefaultValue()!=null).map(p -> p.getConfigKey().getName()).collect(Collectors.toSet());
+        if (child.getConfig().keySet().stream().anyMatch(k -> parentParamsWithDefaults.contains(k.getName()))) {
+            // don't merge if child sets a config value where the parent declares a parameter with a default value
+            // the default value will be clobbered even though someone else might be depending on it
+            return false;
+        }
+
+        // parameters are collapsed on merge so don't need to be considered here
+
+        return true;
+    }
+
+    private static boolean allParentSubsetOfChild(EntitySpec<? extends Entity> spec, EntitySpec<? extends Entity> child, Function<EntitySpec<? extends Entity>, Collection<?>> ...getters) {
+        for (Function<EntitySpec<? extends Entity>, Collection<?>> getter: getters) {
+            if (! (MutableSet.copyOf( getter.apply(child) ).containsAll( getter.apply(spec) )) ) return false;
+        }
+        return true;
     }
 
     public static boolean isWrapperApp(EntitySpec<?> spec) {
