@@ -29,6 +29,8 @@ import org.apache.brooklyn.api.mgmt.Task;
 import org.apache.brooklyn.api.mgmt.entitlement.EntitlementClass;
 import org.apache.brooklyn.api.mgmt.entitlement.EntitlementContext;
 import org.apache.brooklyn.api.mgmt.entitlement.EntitlementManager;
+import org.apache.brooklyn.api.objs.EntityAdjunct;
+import org.apache.brooklyn.api.policy.Policy;
 import org.apache.brooklyn.config.ConfigKey;
 import org.apache.brooklyn.core.config.ConfigKeys;
 import org.apache.brooklyn.core.config.Sanitizer;
@@ -60,23 +62,50 @@ public class Entitlements {
     
     // ------------------- individual permissions
     
+    // TODO applies to bundles and registered types; should pass object or probably better add more entitlements?
     public static EntitlementClass<String> SEE_CATALOG_ITEM = new BasicEntitlementClassDefinition<String>("catalog.see", String.class); 
     public static EntitlementClass<Object> ADD_CATALOG_ITEM = new BasicEntitlementClassDefinition<Object>("catalog.add", Object.class); 
     public static EntitlementClass<StringAndArgument> MODIFY_CATALOG_ITEM = new BasicEntitlementClassDefinition<StringAndArgument>("catalog.modify", StringAndArgument.class); 
     
     public static EntitlementClass<Entity> SEE_ENTITY = new BasicEntitlementClassDefinition<Entity>("entity.see", Entity.class);
+    public static EntitlementClass<Entity> RENAME_ENTITY = new BasicEntitlementClassDefinition<Entity>("entity.rename", Entity.class);
     public static EntitlementClass<EntityAndItem<String>> SEE_SENSOR = new BasicEntitlementClassDefinition<EntityAndItem<String>>("sensor.see", EntityAndItem.typeToken(String.class));
     public static EntitlementClass<EntityAndItem<String>> SEE_CONFIG = new BasicEntitlementClassDefinition<EntityAndItem<String>>("config.see", EntityAndItem.typeToken(String.class));
     public static EntitlementClass<TaskAndItem<String>> SEE_ACTIVITY_STREAMS = new BasicEntitlementClassDefinition<TaskAndItem<String>>("activity.streams.see", TaskAndItem.typeToken(String.class));
     // string is effector name; argument may be a map or a list, depending how the args were supplied
+    // currently this permission gates even _seeing_ the effector; in future we might have a separate permission for that;
+    // this permission also controls setting of _reconfigurable_ config; any other config requires modify entity
     public static EntitlementClass<EntityAndItem<StringAndArgument>> INVOKE_EFFECTOR = new BasicEntitlementClassDefinition<EntityAndItem<StringAndArgument>>("effector.invoke", EntityAndItem.typeToken(StringAndArgument.class));
     public static EntitlementClass<Entity> MODIFY_ENTITY = new BasicEntitlementClassDefinition<Entity>("entity.modify", Entity.class);
-    
+
+    // Adjunct entitlements
+    public static EntitlementClass<EntityAdjunct> DELETE_ADJUNCT = new BasicEntitlementClassDefinition<>("adjunct.delete", EntityAdjunct.class);
+
+    // Location entitlements
+    public static EntitlementClass<StringAndArgument> ADD_LOCATION = new BasicEntitlementClassDefinition<>("location.add", StringAndArgument.class);
+    public static EntitlementClass<StringAndArgument> DELETE_LOCATION = new BasicEntitlementClassDefinition<>("location.delete", StringAndArgument.class);
+    public static EntitlementClass<StringAndArgument> SEE_LOCATION = new BasicEntitlementClassDefinition<>("location.see", StringAndArgument.class);
+
+    // Policy entitlements
+    public static EntitlementClass<StringAndArgument> ADD_POLICY = new BasicEntitlementClassDefinition<>("policy.add", StringAndArgument.class);
+    public static EntitlementClass<Policy> DELETE_POLICY = new BasicEntitlementClassDefinition<>("policy.delete", Policy.class);
+    public static EntitlementClass<Policy> START_POLICY = new BasicEntitlementClassDefinition<>("policy.start", Policy.class);
+    public static EntitlementClass<Policy> STOP_POLICY = new BasicEntitlementClassDefinition<>("policy.stop", Policy.class);
+
+    public static EntitlementClass<Void> SYSTEM_ADMIN = new BasicEntitlementClassDefinition<>("system.admin", Void.class);
+    public static EntitlementClass<Void> HA_STATS = new BasicEntitlementClassDefinition<>("system.ha.stats", Void.class);
+    public static EntitlementClass<Void> HA_ADMIN = new BasicEntitlementClassDefinition<>("system.ha.admin", Void.class);
+    public static EntitlementClass<Void> SHUTDOWN = new BasicEntitlementClassDefinition<>("system.shutdown", Void.class);
+    public static EntitlementClass<Void> USAGE = new BasicEntitlementClassDefinition<>("system.usage", Void.class);
+
+
+
     /**
      * Permission to deploy an application, where parameter is some representation
      * of the app to be deployed (spec instance or yaml plan)
      */
     public static EntitlementClass<Object> DEPLOY_APPLICATION = new BasicEntitlementClassDefinition<Object>("app.deploy", Object.class);
+    public static EntitlementClass<Object> ADD_JAVA = new BasicEntitlementClassDefinition<Object>("java.add", Object.class);
 
     /**
      * Catch-all for catalog, locations, scripting, usage, etc - exporting persistence, shutting down, etc;
@@ -96,6 +125,21 @@ public class Entitlements {
      */
     public static EntitlementClass<Void> ROOT = new BasicEntitlementClassDefinition<Void>("root", Void.class);
 
+    /**
+     * Permission to query the query the log store
+     */
+    public static EntitlementClass<Void> LOGBOOK_LOG_STORE_QUERY = new BasicEntitlementClassDefinition<Void>("logbook.query", Void.class);
+
+    /**
+     * Permission to execute groovy scripts. Since 1.1 prefer {@link #EXECUTE_SCRIPT}.
+     */
+    public static EntitlementClass<Void> EXECUTE_GROOVY_SCRIPT = new BasicEntitlementClassDefinition<Void>("groovy_script.execute", Void.class);
+
+    /**
+     * Permission to run local script commands, eg 'shell' workflow step, groovy scripts, etc
+     */
+    public static EntitlementClass<Void> EXECUTE_SCRIPT = new BasicEntitlementClassDefinition<Void>("script.execute", Void.class);
+
     @SuppressWarnings("unchecked")
     public enum EntitlementClassesEnum {
         ENTITLEMENT_SEE_CATALOG_ITEM(SEE_CATALOG_ITEM) { public <T> T handle(EntitlementClassesHandler<T> handler, Object argument) { return handler.handleSeeCatalogItem((String)argument); } },
@@ -108,10 +152,17 @@ public class Entitlements {
         ENTITLEMENT_MODIFY_ENTITY(MODIFY_ENTITY) { public <T> T handle(EntitlementClassesHandler<T> handler, Object argument) { return handler.handleModifyEntity((Entity)argument); } },
         
         ENTITLEMENT_DEPLOY_APPLICATION(DEPLOY_APPLICATION) { public <T> T handle(EntitlementClassesHandler<T> handler, Object argument) { return handler.handleDeployApplication(argument); } },
-        
+
+        ENTITLEMENT_ADD_JAVA(ADD_JAVA) { public <T> T handle(EntitlementClassesHandler<T> handler, Object argument) { return handler.handleAddJava(argument); } },
+
         ENTITLEMENT_SEE_ALL_SERVER_INFO(SEE_ALL_SERVER_INFO) { public <T> T handle(EntitlementClassesHandler<T> handler, Object argument) { return handler.handleSeeAllServerInfo(); } },
         ENTITLEMENT_SERVER_STATUS(SERVER_STATUS) { public <T> T handle(EntitlementClassesHandler<T> handler, Object argument) { return handler.handleSeeServerStatus(); } },
         ENTITLEMENT_ROOT(ROOT) { public <T> T handle(EntitlementClassesHandler<T> handler, Object argument) { return handler.handleRoot(); } },
+        ENTITLEMENT_EXECUTE_GROOVY_SCRIPT(EXECUTE_GROOVY_SCRIPT) { public <T> T handle(EntitlementClassesHandler<T> handler, Object argument) { return handler.handleExecuteGroovyScript(); } },
+        ENTITLEMENT_EXECUTE_SCRIPT(EXECUTE_SCRIPT) { public <T> T handle(EntitlementClassesHandler<T> handler, Object argument) { return handler.handleExecuteScript(); } },
+
+        /* NOTE, 'ROOT' USER ONLY IS ALLOWED TO SEE THE LOGS. */
+        ENTITLEMENT_LOGBOOK_QUERY(LOGBOOK_LOG_STORE_QUERY) { public <T> T handle(EntitlementClassesHandler<T> handler, Object argument) { return handler.handleRoot(); } },
         ;
         
         private EntitlementClass<?> entitlementClass;
@@ -143,7 +194,10 @@ public class Entitlements {
         public T handleInvokeEffector(EntityAndItem<StringAndArgument> effectorInfo);
         public T handleModifyEntity(Entity entity);
         public T handleDeployApplication(Object app);
+        public T handleAddJava(Object app);
         public T handleSeeAllServerInfo();
+        public T handleExecuteGroovyScript();
+        public T handleExecuteScript();
         public T handleRoot();
     }
     
@@ -220,13 +274,37 @@ public class Entitlements {
     }
 
     /**
-     * @return An entitlement manager allowing everything but {@link #ROOT} and {@link #SEE_ALL_SERVER_INFO}.
+     * @return An entitlement manager allowing everything but {@link #EXECUTE_SCRIPT}, {@link #EXECUTE_GROOVY_SCRIPT}.
+     */
+    public static EntitlementManager powerUser() {
+        return new EntitlementManager() {
+            @Override
+            public <T> boolean isEntitled(EntitlementContext context, EntitlementClass<T> permission, T entitlementClassArgument) {
+                return !EXECUTE_GROOVY_SCRIPT.equals(permission) && !EXECUTE_SCRIPT.equals(permission);
+            }
+            @Override
+            public String toString() {
+                return "Entitlements.powerUser";
+            }
+        };
+    }
+
+    /**
+     * @return An entitlement manager allowing everything but {@link #ROOT}, {@link #LOGBOOK_LOG_STORE_QUERY}, {@link #SEE_ALL_SERVER_INFO},
+     * {@link #EXECUTE_GROOVY_SCRIPT}, {@link #EXECUTE_SCRIPT}, {@link #MODIFY_ENTITY}, and {@link #HA_ADMIN}.
      */
     public static EntitlementManager user() {
         return new EntitlementManager() {
             @Override
             public <T> boolean isEntitled(EntitlementContext context, EntitlementClass<T> permission, T entitlementClassArgument) {
-                return !SEE_ALL_SERVER_INFO.equals(permission) && !ROOT.equals(permission);
+                return
+                        !SEE_ALL_SERVER_INFO.equals(permission) &&
+                        !ROOT.equals(permission) &&
+                        !LOGBOOK_LOG_STORE_QUERY.equals(permission) &&
+                        !EXECUTE_GROOVY_SCRIPT.equals(permission) &&
+                        !EXECUTE_SCRIPT.equals(permission) &&
+                        !MODIFY_ENTITY.equals(permission) &&
+                        !HA_ADMIN.equals(permission);
             }
             @Override
             public String toString() {
@@ -235,6 +313,35 @@ public class Entitlements {
         };
     }
 
+    /**
+     * @return An entitlement manager per {@link #user()} but also disallowing {@link #ADD_JAVA}
+     */
+    public static EntitlementManager blueprintAuthor() {
+        return new EntitlementManager() {
+            @Override
+            public <T> boolean isEntitled(EntitlementContext context, EntitlementClass<T> permission, T entitlementClassArgument) {
+                return user().isEntitled(context, permission, entitlementClassArgument) && !ADD_JAVA.equals(permission);
+            }
+            @Override
+            public String toString() {
+                return "Entitlements.user";
+            }
+        };
+    }
+
+    public static EntitlementManager logViewer() {
+        return new EntitlementManager() {
+            @Override
+            public <T> boolean isEntitled(EntitlementContext context, EntitlementClass<T> permission, T entitlementClassArgument) {
+                return LOGBOOK_LOG_STORE_QUERY.equals(permission);
+            }
+            @Override
+            public String toString() {
+                return "Entitlements.logViewer";
+            }
+        };
+    }
+
     /**
      * @return An entitlement manager denying access to anything that requires entitlements.
      */
@@ -350,6 +457,9 @@ public class Entitlements {
             FineGrainedEntitlements.allowing(SEE_ENTITY),
             FineGrainedEntitlements.allowing(SEE_ACTIVITY_STREAMS),
             FineGrainedEntitlements.allowing(SEE_CATALOG_ITEM),
+            FineGrainedEntitlements.allowing(SERVER_STATUS),
+            FineGrainedEntitlements.allowing(SEE_LOCATION),
+            FineGrainedEntitlements.allowing(HA_STATS),
             FineGrainedEntitlements.seeNonSecretSensors(),
             FineGrainedEntitlements.seeNonSecretConfig()
         );
@@ -392,6 +502,19 @@ public class Entitlements {
         return null;
     }
 
+    public static String getEntitlementContextUser() {
+        return getEntitlementContextUserMaybe().or("<system>");
+    }
+
+    public static Maybe<String> getEntitlementContextUserMaybe() {
+        EntitlementContext ctx = getEntitlementContext();
+        if (ctx!=null) {
+            String user = ctx.user();
+            if (Strings.isNonBlank(user)) return Maybe.of(user);
+        }
+        return Maybe.absent();
+    }
+
     public static void setEntitlementContext(EntitlementContext context) {
         EntitlementContext oldContext = PerThreadEntitlementContextHolder.perThreadEntitlementsContextHolder.get();
         if (oldContext!=null && context!=null) {
@@ -415,15 +538,6 @@ public class Entitlements {
             throw new NotEntitledException(getEntitlementContext(), permission, typeArgument);
         }
     }
-    /** throws {@link NotEntitledException} if entitlement not available for current {@link #getEntitlementContext()} 
-     * @since 0.7.0
-     * @deprecated since 0.7.0, use {@link #checkEntitled(EntitlementManager, EntitlementClass, Object)};
-     * kept briefly because there is some downstream usage*/
-    // Note: @Deprecated annotation only added from v0.10.0.
-    @Deprecated
-    public static <T> void requireEntitled(EntitlementManager checker, EntitlementClass<T> permission, T typeArgument) {
-        checkEntitled(checker, permission, typeArgument);
-    }
     
     // ----------------- initialization ----------------
 
@@ -451,6 +565,12 @@ public class Entitlements {
             return minimal();
         } else if ("user".equalsIgnoreCase(type)) {
             return user();
+        } else if ("powerUser".equalsIgnoreCase(type) || "power_user".equalsIgnoreCase(type)) {
+            return powerUser();
+        } else if ("blueprintAuthor".equalsIgnoreCase(type) || "blueprint_author".equalsIgnoreCase(type)) {
+            return blueprintAuthor();
+        }else if ("logViewer".equalsIgnoreCase(type) || "log_viewer".equalsIgnoreCase(type) || "log".equalsIgnoreCase(type)) {
+            return logViewer();
         }
         if (Strings.isNonBlank(type)) {
             try {
