@@ -41,19 +41,26 @@ public class BasicManagedBundleMemento extends AbstractMemento implements Manage
     public static class Builder extends AbstractMemento.Builder<Builder> {
         protected String symbolicName;
         protected String version;
+        protected String format;
         protected String url;
         protected String checksum;
-        
+        protected Boolean deleteable;
+
         public Builder symbolicName(String symbolicName) {
             this.symbolicName = symbolicName;
             return self();
         }
-        
+
         public Builder version(String version) {
             this.version = version;
             return self();
         }
 
+        public Builder format(String format) {
+            this.format = format;
+            return self();
+        }
+
         public Builder url(String url) {
             this.url = url;
             return self();
@@ -64,12 +71,19 @@ public class BasicManagedBundleMemento extends AbstractMemento implements Manage
             return self();
         }
 
+        public Builder deleteable(Boolean deleteable) {
+            this.deleteable = deleteable;
+            return self();
+        }
+
         public Builder from(ManagedBundleMemento other) {
             super.from(other);
             symbolicName = other.getSymbolicName();
             version = other.getVersion();
+            format = other.getFormat();
             url = other.getUrl();
             checksum = other.getChecksum();
+            deleteable = other.getDeleteable();
             return self();
         }
 
@@ -80,8 +94,10 @@ public class BasicManagedBundleMemento extends AbstractMemento implements Manage
 
     private String symbolicName;
     private String version;
+    private String format;
     private String url;
     private String checksum;
+    private Boolean deleteable;
     transient private ByteSource jarContent;
 
     @SuppressWarnings("unused") // For deserialisation
@@ -91,8 +107,10 @@ public class BasicManagedBundleMemento extends AbstractMemento implements Manage
         super(builder);
         this.symbolicName = builder.symbolicName;
         this.version = builder.version;
+        this.format = builder.format;
         this.url = builder.url;
         this.checksum = builder.checksum;
+        this.deleteable = builder.deleteable;
     }
 
     @Override
@@ -105,6 +123,11 @@ public class BasicManagedBundleMemento extends AbstractMemento implements Manage
         return version;
     }
 
+    @Override
+    public String getFormat() {
+        return format;
+    }
+
     @Override
     public String getUrl() {
         return url;
@@ -115,11 +138,14 @@ public class BasicManagedBundleMemento extends AbstractMemento implements Manage
         return checksum;
     }
 
+    @Override
+    public Boolean getDeleteable() { return deleteable; }
+
     @Override
     public ByteSource getJarContent() {
         return jarContent;
     }
-    
+
     @Override
     public void setJarContent(ByteSource byteSource) {
         this.jarContent = byteSource;
@@ -143,7 +169,16 @@ public class BasicManagedBundleMemento extends AbstractMemento implements Manage
         return super.newVerboseStringHelper()
                 .add("symbolicName", getSymbolicName())
                 .add("version", getVersion())
+                .add("format", getFormat())
                 .add("url", getUrl())
-                .add("checksum", getChecksum());
+                .add("checksum", getChecksum())
+                .add("deleteable", getDeleteable());
     }
+
+    @Override
+    public String toString() {
+        // include more details on toString here, so we can see what/where it is being persisted
+        return toVerboseString();
+    }
+
 }
