@@ -18,14 +18,11 @@
  */
 package org.apache.brooklyn.core.objs;
 
-import static com.google.common.base.Preconditions.checkArgument;
-
 import java.lang.reflect.Field;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
-import java.util.Date;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -54,9 +51,10 @@ import org.apache.brooklyn.core.sensor.PortAttributeSensorAndConfigKey;
 import org.apache.brooklyn.util.collections.MutableList;
 import org.apache.brooklyn.util.collections.MutableMap;
 import org.apache.brooklyn.util.collections.MutableSet;
+import org.apache.brooklyn.util.core.flags.BrooklynTypeNameResolution;
+import org.apache.brooklyn.util.core.flags.TypeCoercions;
 import org.apache.brooklyn.util.guava.Maybe;
-import org.apache.brooklyn.util.text.StringPredicates;
-import org.apache.brooklyn.util.time.Duration;
+import org.apache.brooklyn.util.guava.TypeTokens;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -70,14 +68,13 @@ import com.google.common.base.Predicates;
 import com.google.common.collect.FluentIterable;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
-import com.google.common.collect.Iterables;
 import com.google.common.reflect.TypeToken;
 
 public class BasicSpecParameter<T> implements SpecParameter<T>{
     private static final long serialVersionUID = -4728186276307619778L;
 
     private static final Logger log = LoggerFactory.getLogger(BasicSpecParameter.class);
-    
+        
     private final String label;
 
     /** pinning may become a priority or other more expansive indicator */
@@ -202,37 +199,6 @@ public class BasicSpecParameter<T> implements SpecParameter<T>{
     }
 
     private static final class ParseYamlInputs {
-        private static final String DEFAULT_TYPE = "string";
-        private static final Map<String, Class<?>> BUILT_IN_TYPES = ImmutableMap.<String, Class<?>>builder()
-                .put(DEFAULT_TYPE, String.class)
-                .put("bool", Boolean.class)
-                .put("boolean", Boolean.class)
-                .put("byte", Byte.class)
-                .put("char", Character.class)
-                .put("character", Character.class)
-                .put("short", Short.class)
-                .put("integer", Integer.class)
-                .put("int", Integer.class)
-                .put("long", Long.class)
-                .put("float", Float.class)
-                .put("double", Double.class)
-                .put("duration", Duration.class)
-                .put("timestamp", Date.class)
-                .put("port", PortRange.class)
-                .build();
-
-        private static final Map<String, Predicate<?>> BUILT_IN_CONSTRAINTS = ImmutableMap.<String, Predicate<?>>of(
-                "required", StringPredicates.isNonBlank());
-        
-        private static final Map<String, Function<Object, Predicate<?>>> BUILT_IN_CONSTRAINT_FACTORIES = ImmutableMap.<String, Function<Object, Predicate<?>>>of(
-                "regex", new Function<Object, Predicate<?>>() {
-                    @Override public Predicate<?> apply(Object input) {
-                        // TODO Could try to handle deferred supplier as well?
-                        checkArgument(input instanceof String, "Constraint regex value must be a string, but got %s (%s)",
-                                (input == null ? "null" : input.getClass().getName()), input);
-                        return StringPredicates.matchesRegex((String)input);
-                    }});
-        
         private static List<SpecParameter<?>> parseParameters(List<?> inputsRaw, Function<Object, Object> specialFlagTransformer, BrooklynClassLoadingContext loader) {
             if (inputsRaw == null) return ImmutableList.of();
             List<SpecParameter<?>> inputs = new ArrayList<>(inputsRaw.size());
@@ -283,6 +249,8 @@ public class BasicSpecParameter<T> implements SpecParameter<T>{
             boolean hasTypeInheritance = inputDef.containsKey("inheritance.type");
             ConfigInheritance typeInheritance = parseInheritance(inputDef.get("inheritance.type"), loader);
 
+            Boolean isReconfigurable = TypeCoercions.coerce(inputDef.get("reconfigurable"), Boolean.class);
+
             if (name == null) {
                 throw new IllegalArgumentException("'name' value missing from input definition " + obj + " but is required. Check for typos.");
             }
@@ -292,7 +260,7 @@ public class BasicSpecParameter<T> implements SpecParameter<T>{
 
             boolean hasType = type!=null;
 
-            TypeToken typeToken = inferType(type, loader);
+            TypeToken typeToken = resolveType(type, loader);
             Object immutableDefaultValue = tryToImmutable(defaultValue, typeToken);
 
             Builder builder = BasicConfigKey.builder(typeToken)
@@ -303,8 +271,9 @@ public class BasicSpecParameter<T> implements SpecParameter<T>{
                     .constraint(constraint)
                     .runtimeInheritance(runtimeInheritance)
                     .typeInheritance(typeInheritance);
+            if (isReconfigurable!=null) builder.reconfigurable(isReconfigurable);
 
-            if (PortRange.class.equals(typeToken.getRawType())) {
+            if (TypeTokens.equalsRaw(PortRange.class, typeToken)) {
                 sensorType = new PortAttributeSensorAndConfigKey(builder);
                 configType = ((HasConfigKey)sensorType).getConfigKey();
             } else {
@@ -317,7 +286,7 @@ public class BasicSpecParameter<T> implements SpecParameter<T>{
 
         private static Object tryToImmutable(Object val, TypeToken<?> type) {
             Object result;
-            if (Set.class.isAssignableFrom(type.getRawType()) && val instanceof Iterable) {
+            if (TypeTokens.isAssignableFromRaw(Set.class, type) && val instanceof Iterable) {
                 result = Collections.unmodifiableSet(MutableSet.copyOf((Iterable<?>)val));
             } else if (val instanceof Iterable) {
                 result = Collections.unmodifiableList(MutableList.copyOf((Iterable<?>)val));
@@ -327,29 +296,22 @@ public class BasicSpecParameter<T> implements SpecParameter<T>{
                 return val;
             }
             
-            if (type != null && !type.isAssignableFrom(result.getClass())) {
-                log.warn("Unable to convert parameter default value (type "+type+") to immutable");
+            if (type != null && !TypeTokens.isInstanceRaw(type, result)) {
+                // a Map is not the desired type, but val is a Map that will likely be coerced to the type later
+                // in case val is a special subtype of map, do not return unmodifiableMap but preffer the actual defined instance
+                // there is a chance a developer might change val because it is not mutable
+                // however there are plenty of valid paths to get here
+                log.trace("Unable to convert parameter default value (type "+type+") to immutable");
                 return val;
             } else {
                 return result;
             }
         }
         
-        @SuppressWarnings({ "rawtypes" })
-        private static TypeToken inferType(String typeRaw, BrooklynClassLoadingContext loader) {
+        private static TypeToken<?> resolveType(String typeRaw, BrooklynClassLoadingContext loader) {
             if (typeRaw == null) return TypeToken.of(String.class);
-            String type = typeRaw.trim();
-            if (BUILT_IN_TYPES.containsKey(type.toLowerCase())) {
-                return TypeToken.of(BUILT_IN_TYPES.get(type.toLowerCase()));
-            } else {
-                // Assume it's a Java type
-                Maybe<Class<?>> inputType = loader.tryLoadClass(type);
-                if (inputType.isPresent()) {
-                    return TypeToken.of(inputType.get());
-                } else {
-                    throw new IllegalArgumentException("The type '" + type + "' for a catalog input not recognised as a built-in (" + BUILT_IN_TYPES.keySet() + ") or a java type");
-                }
-            }
+            return new BrooklynTypeNameResolution.BrooklynTypeNameResolver("parameter", loader, true, true)
+                    .getTypeToken(typeRaw);
         }
 
         @SuppressWarnings({ "unchecked", "rawtypes" })
@@ -381,40 +343,7 @@ public class BasicSpecParameter<T> implements SpecParameter<T>{
         }
 
         private static Predicate<?> parseConstraint(Object untypedConstraint, BrooklynClassLoadingContext loader) {
-            // TODO Could try to handle deferred supplier as well?
-            if (untypedConstraint instanceof Predicate) {
-                // An explicit predicate (e.g. via "$brooklyn:object: ...")
-                return (Predicate<?>) untypedConstraint;
-            } else if (untypedConstraint instanceof String) {
-                // build-in simple declaration, such as "required"
-                String constraint = (String)untypedConstraint;
-                if (BUILT_IN_CONSTRAINTS.containsKey(constraint)) {
-                    return BUILT_IN_CONSTRAINTS.get(constraint);
-                } else {
-                    throw new IllegalArgumentException("The constraint '" + constraint + "' for a catalog input is not "
-                            + "recognized as a built-in (" + BUILT_IN_CONSTRAINTS.keySet() + " or " 
-                            + BUILT_IN_CONSTRAINT_FACTORIES.keySet() + ")");
-                }
-            } else if (untypedConstraint instanceof Map) {
-                // For example "regex: foo.*"
-                Map<?,?> constraint = (Map<?,?>)untypedConstraint;
-                if (constraint.size() == 1) {
-                    Object key = Iterables.getOnlyElement(constraint.keySet());
-                    Object val = constraint.get(key);
-                    if (BUILT_IN_CONSTRAINT_FACTORIES.containsKey(key)) {
-                        Function<Object, Predicate<?>> factory = BUILT_IN_CONSTRAINT_FACTORIES.get(key);
-                        return factory.apply(val);
-                    } else {
-                        throw new IllegalArgumentException("The constraint '" + constraint + "' for a catalog input is not "
-                                + "recognized as a built-in (" + BUILT_IN_CONSTRAINTS.keySet() + ")");
-                    }
-                } else {
-                    throw new IllegalArgumentException("The config key constraint '" + constraint + "' is not supported - "
-                            + "it can handle only single key:value constraint.");
-                }
-            } else {
-                throw new IllegalArgumentException("The constraint '" + untypedConstraint + "' for a catalog input is not recognized");
-            }
+            return ConstraintSerialization.INSTANCE.toPredicateFromJson(untypedConstraint);
         }
         
         private static ConfigInheritance parseInheritance(Object obj, BrooklynClassLoadingContext loader) {
@@ -503,7 +432,7 @@ public class BasicSpecParameter<T> implements SpecParameter<T>{
      * {@link AbstractBrooklynObjectSpec spec}, and if spec has no parameters it 
      * also generates a list from the spec
      *
-     * @see EntitySpec#parameters(List)
+     * @see EntitySpec#parameters(Iterable)
      */
     @Beta
     public static void initializeSpecWithExplicitParameters(AbstractBrooklynObjectSpec<?, ?> spec, List<? extends SpecParameter<?>> explicitParams, BrooklynClassLoadingContext loader) {
@@ -534,7 +463,8 @@ public class BasicSpecParameter<T> implements SpecParameter<T>{
 
         if (newParams!=null) {
             for (SpecParameter<?> p: newParams) {
-                final SpecParameter<?> existingP = existingToKeep.remove(p.getConfigKey().getName());
+                // Don't remove + add, as that changes order: see https://issues.apache.org/jira/browse/BROOKLYN-602
+                final SpecParameter<?> existingP = existingToKeep.get(p.getConfigKey().getName());
                 if (p instanceof SpecParameterIncludingDefinitionForInheritance) {
                     // config keys should be transformed to parameters and so should be found;
                     // so the code below is correct whether existingP is set or is null 
@@ -543,7 +473,12 @@ public class BasicSpecParameter<T> implements SpecParameter<T>{
                     // shouldn't happen; all calling logic should get SpecParameterForInheritance;
                     log.warn("Found non-definitional spec parameter: "+p+" adding to "+spec);
                 }
-                result.add(p);
+                
+                if (existingP != null) {
+                    existingToKeep.put(p.getConfigKey().getName(), p);
+                } else {
+                    result.add(p);
+                }
             }
         }
 
@@ -552,7 +487,7 @@ public class BasicSpecParameter<T> implements SpecParameter<T>{
     }
 
     /** instance of {@link SpecParameter} which includes information about what fields are explicitly set,
-     * for use with a subsequent merge with ancestor config keys; see {@link #resolveParameters(Collection, Collection, AbstractBrooklynObjectSpec)}*/
+     * for use with a subsequent merge with ancestor config keys; see {@link #resolveParameters(Collection, AbstractBrooklynObjectSpec)}*/
     @SuppressWarnings("serial")
     @Beta
     static class SpecParameterIncludingDefinitionForInheritance<T> extends BasicSpecParameter<T> {
