@@ -18,20 +18,11 @@
  */
 package org.apache.brooklyn.feed.http;
 
-import static com.google.common.base.Preconditions.checkNotNull;
-
-import java.io.ByteArrayOutputStream;
-import java.io.IOException;
-import java.net.URI;
-import java.net.URL;
-import java.nio.charset.StandardCharsets;
-import java.util.Collection;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
-import java.util.concurrent.Callable;
-import java.util.concurrent.TimeUnit;
-
+import com.google.common.base.*;
+import com.google.common.collect.*;
+import com.google.common.io.BaseEncoding;
+import com.google.common.io.ByteStreams;
+import com.google.common.reflect.TypeToken;
 import org.apache.brooklyn.api.entity.Entity;
 import org.apache.brooklyn.api.entity.EntityLocal;
 import org.apache.brooklyn.api.location.Location;
@@ -46,15 +37,16 @@ import org.apache.brooklyn.core.feed.Poller;
 import org.apache.brooklyn.core.location.Locations;
 import org.apache.brooklyn.core.location.Machines;
 import org.apache.brooklyn.core.location.internal.LocationInternal;
+import org.apache.brooklyn.core.sensor.AbstractAddTriggerableSensor;
+import org.apache.brooklyn.feed.function.FunctionFeed;
+import org.apache.brooklyn.util.core.javalang.BrooklynHttpConfig;
 import org.apache.brooklyn.util.executor.HttpExecutorFactory;
 import org.apache.brooklyn.util.guava.Maybe;
 import org.apache.brooklyn.util.http.HttpToolResponse;
-import org.apache.brooklyn.util.http.executor.HttpConfig;
+import org.apache.brooklyn.util.http.auth.UsernamePassword;
 import org.apache.brooklyn.util.http.executor.HttpExecutor;
 import org.apache.brooklyn.util.http.executor.HttpRequest;
 import org.apache.brooklyn.util.http.executor.HttpResponse;
-import org.apache.brooklyn.util.http.auth.UsernamePassword;
-import org.apache.brooklyn.util.http.executor.apacheclient.HttpExecutorImpl;
 import org.apache.brooklyn.util.stream.Streams;
 import org.apache.brooklyn.util.time.Duration;
 import org.apache.http.auth.Credentials;
@@ -62,20 +54,19 @@ import org.apache.http.auth.UsernamePasswordCredentials;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import com.google.common.base.Objects;
-import com.google.common.base.Optional;
-import com.google.common.base.Supplier;
-import com.google.common.base.Suppliers;
-import com.google.common.base.Throwables;
-import com.google.common.collect.HashMultimap;
-import com.google.common.collect.ImmutableMap;
-import com.google.common.collect.Lists;
-import com.google.common.collect.Maps;
-import com.google.common.collect.SetMultimap;
-import com.google.common.collect.Sets;
-import com.google.common.io.BaseEncoding;
-import com.google.common.io.ByteStreams;
-import com.google.common.reflect.TypeToken;
+import java.io.ByteArrayOutputStream;
+import java.io.IOException;
+import java.net.URI;
+import java.net.URL;
+import java.nio.charset.StandardCharsets;
+import java.util.Collection;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
+import java.util.concurrent.Callable;
+import java.util.concurrent.TimeUnit;
+
+import static com.google.common.base.Preconditions.checkNotNull;
 
 /**
  * Provides a feed of attribute values, by polling over http.
@@ -135,6 +126,7 @@ public class HttpFeed extends AbstractFeed {
     }
     
     public static class Builder {
+        private String name;
         private Entity entity;
         private boolean onlyIfServiceUp = false;
         private Supplier<URI> baseUriProvider;
@@ -150,6 +142,10 @@ public class HttpFeed extends AbstractFeed {
         private Boolean preemptiveBasicAuth;
         private volatile boolean built;
 
+        public Builder name(String val) {
+            this.name = val;
+            return this;
+        }
         public Builder entity(Entity val) {
             this.entity = val;
             return this;
@@ -263,11 +259,20 @@ public class HttpFeed extends AbstractFeed {
             }
         }
         public HttpFeed build() {
+            return build(false, null);
+        }
+
+        /** normally no arg needed, but can pass true for first to register, and null for latter to auto-start at the right time, or true to force start */
+        public HttpFeed build(boolean registerOnEntity, Boolean feedStart) {
             built = true;
             HttpFeed result = new HttpFeed(this);
             result.setEntity(checkNotNull((EntityLocal)entity, "entity"));
             if (suspended) result.suspend();
-            result.start();
+            if (registerOnEntity) entity.addFeed(result);
+            if ((feedStart==null && Entities.isManagedActive(entity)) || Boolean.TRUE.equals(feedStart)) {
+                // this feed is used a lot without being attached to an entity, not ideal, but let's support it
+                result.start();
+            }
             return result;
         }
         @Override
@@ -331,6 +336,8 @@ public class HttpFeed extends AbstractFeed {
     }
     
     protected HttpFeed(Builder builder) {
+        if (builder.name!=null) setDisplayName(builder.name);
+
         config().set(ONLY_IF_SERVICE_UP, builder.onlyIfServiceUp);
         Map<String,String> baseHeaders = builder.buildBaseHeaders();
 
@@ -346,7 +353,8 @@ public class HttpFeed extends AbstractFeed {
                 Map<String, Object> httpExecutorProps = ((LocationInternal)location.get()).config().getBag().getAllConfig();
                 httpExecutor = httpExecutorFactory.getHttpExecutor(httpExecutorProps);
             } else {
-                httpExecutor = HttpExecutorImpl.newInstance();
+                httpExecutor = BrooklynHttpConfig.newHttpExecutor(
+                        Preconditions.checkNotNull(location.isPresent() ? location.get() : builder.entity, "feed requires a location or entity") );
             }
         }
 
@@ -384,54 +392,27 @@ public class HttpFeed extends AbstractFeed {
 
     @Override
     protected void preStart() {
-        SetMultimap<HttpPollIdentifier, HttpPollConfig<?>> polls = getConfig(POLLS);
+        getPoller().scheduleFeed(this, getConfig(POLLS), pollInfo -> () -> {
+                if (log.isTraceEnabled()) log.trace("http polling for {} sensors at {}", entity, pollInfo);
 
-        for (final HttpPollIdentifier pollInfo : polls.keySet()) {
-            // Though HttpClients are thread safe and can take advantage of connection pooling
-            // and authentication caching, the httpcomponents documentation says:
-            //    "While HttpClient instances are thread safe and can be shared between multiple
-            //     threads of execution, it is highly recommended that each thread maintains its
-            //     own dedicated instance of HttpContext.
-            //  http://hc.apache.org/httpcomponents-client-ga/tutorial/html/connmgmt.html
+                UsernamePassword creds = null;
+                if (pollInfo.credentials.isPresent()) {
+                    creds = new UsernamePassword(
+                            pollInfo.credentials.get().getUserPrincipal().getName(),
+                            pollInfo.credentials.get().getPassword());
+                }
 
-            Set<HttpPollConfig<?>> configs = polls.get(pollInfo);
-            long minPeriod = Integer.MAX_VALUE;
-            Set<AttributePollHandler<? super HttpToolResponse>> handlers = Sets.newLinkedHashSet();
-
-            for (HttpPollConfig<?> config : configs) {
-                handlers.add(new AttributePollHandler<HttpToolResponse>(config, entity, this));
-                if (config.getPeriod() > 0) minPeriod = Math.min(minPeriod, config.getPeriod());
-            }
-
-            Callable<HttpToolResponse> pollJob;
-            pollJob = new Callable<HttpToolResponse>() {
-                @Override
-                public HttpToolResponse call() throws Exception {
-                    if (log.isTraceEnabled()) log.trace("http polling for {} sensors at {}", entity, pollInfo);
-
-                    UsernamePassword creds = null;
-                    if (pollInfo.credentials.isPresent()) {
-                        creds =  new UsernamePassword(
-                                pollInfo.credentials.get().getUserPrincipal().getName(),
-                                pollInfo.credentials.get().getPassword());
-                    }
-
-                    HttpResponse response =  pollInfo.httpExecutor.execute(new HttpRequest.Builder()
-                            .headers(pollInfo.headers)
-                            .uri(pollInfo.uriProvider.get())
-                            .credentials(creds)
-                            .method(pollInfo.method)
-                            .body(pollInfo.body)
-                            .config(HttpConfig.builder()
-                                    .trustSelfSigned(true)
-                                    .trustAll(true)
-                                    .laxRedirect(true)
-                                    .build())
-                            .build());
-                    return createHttpToolRespose(response);
-                }};
-                getPoller().scheduleAtFixedRate(pollJob, new DelegatingPollHandler<HttpToolResponse>(handlers), minPeriod);
-        }
+                final long startTime = System.currentTimeMillis();
+                HttpResponse response = pollInfo.httpExecutor.execute(new HttpRequest.Builder()
+                        .headers(pollInfo.headers)
+                        .uri(pollInfo.uriProvider.get())
+                        .credentials(creds)
+                        .method(pollInfo.method)
+                        .body(pollInfo.body)
+                        .config(BrooklynHttpConfig.httpConfigBuilder(getEntity()).build())
+                        .build());
+                return createHttpToolRespose(response, startTime);
+        });
     }
 
     @Override
@@ -441,7 +422,7 @@ public class HttpFeed extends AbstractFeed {
     }
 
     @SuppressWarnings("unchecked")
-    private HttpToolResponse createHttpToolRespose(HttpResponse response) throws IOException {
+    private HttpToolResponse createHttpToolRespose(HttpResponse response, long startTime) throws IOException {
         int responseCode = response.code();
 
         Map<String,? extends List<String>> headers = (Map<String, List<String>>) (Map<?, ?>) response.headers().asMap();
@@ -449,7 +430,6 @@ public class HttpFeed extends AbstractFeed {
         byte[] content = null;
         final long durationMillisOfFirstResponse;
         final long durationMillisOfFullContent;
-        final long startTime = System.currentTimeMillis();
 
         ByteArrayOutputStream out = new ByteArrayOutputStream();
         durationMillisOfFirstResponse = Duration.sinceUtc(startTime).toMilliseconds();
