@@ -75,6 +75,7 @@ import org.apache.brooklyn.util.core.ResourceUtils;
 import org.apache.brooklyn.util.core.config.ConfigBag;
 import org.apache.brooklyn.util.core.crypto.SecureKeys;
 import org.apache.brooklyn.util.core.file.ArchiveUtils;
+import org.apache.brooklyn.util.core.file.BrooklynOsCommands;
 import org.apache.brooklyn.util.core.flags.SetFromFlag;
 import org.apache.brooklyn.util.core.flags.TypeCoercions;
 import org.apache.brooklyn.util.core.internal.ssh.ShellTool;
@@ -92,7 +93,7 @@ import org.apache.brooklyn.util.guava.KeyTransformingLoadingCache.KeyTransformin
 import org.apache.brooklyn.util.guava.Maybe;
 import org.apache.brooklyn.util.pool.BasicPool;
 import org.apache.brooklyn.util.pool.Pool;
-import org.apache.brooklyn.util.ssh.BashCommands;
+import org.apache.brooklyn.util.ssh.BashCommandsConfigurable;
 import org.apache.brooklyn.util.stream.KnownSizeInputStream;
 import org.apache.brooklyn.util.stream.ReaderInputStream;
 import org.apache.brooklyn.util.stream.StreamGobbler;
@@ -288,7 +289,7 @@ public class SshMachineLocation extends AbstractMachineLocation implements Machi
                 if (!publicEndpoint.hasPort()) {
                     throw new IllegalArgumentException("Invalid portMapping ('"+entry.getValue()+"') for port "+targetPort+" in machine "+this);
                 }
-                pfm.associate(publicEndpoint.getHostText(), publicEndpoint, this, targetPort);
+                pfm.associate(publicEndpoint.getHost(), publicEndpoint, this, targetPort);
             }
         }
     }
@@ -514,6 +515,7 @@ public class SshMachineLocation extends AbstractMachineLocation implements Machi
         return HostAndPort.fromParts(host, port);
     }
 
+    @Override
     public String getUser() {
         if (!groovyTruth(user)) {
             if (config().getLocalRaw(SshTool.PROP_USER).isPresent()) {
@@ -701,13 +703,14 @@ public class SshMachineLocation extends AbstractMachineLocation implements Machi
     public int execCommands(String summaryForLogging, List<String> commands) {
         return execCommands(MutableMap.<String,Object>of(), summaryForLogging, commands, MutableMap.<String,Object>of());
     }
-    public int execCommands(Map<String,?> props, String summaryForLogging, List<String> commands) {
+    public int execCommands(Map<String, ?> props, String summaryForLogging, List<String> commands) {
         return execCommands(props, summaryForLogging, commands, MutableMap.<String,Object>of());
     }
-    public int execCommands(String summaryForLogging, List<String> commands, Map<String,?> env) {
+    public int execCommands(String summaryForLogging, List<String> commands, Map<String, ?> env) {
         return execCommands(MutableMap.<String,Object>of(), summaryForLogging, commands, env);
     }
-    public int execCommands(Map<String,?> props, String summaryForLogging, List<String> commands, Map<String,?> env) {
+    @Override
+    public int execCommands(Map<String, ?> props, String summaryForLogging, List<String> commands, Map<String, ?> env) {
         return newExecWithLoggingHelpers().execCommands(augmentPropertiesWithSshConfigGivenToProps(props), summaryForLogging, commands, env);
     }
 
@@ -727,6 +730,7 @@ public class SshMachineLocation extends AbstractMachineLocation implements Machi
     public int execScript(String summaryForLogging, List<String> commands, Map<String,?> env) {
         return execScript(MutableMap.<String,Object>of(), summaryForLogging, commands, env);
     }
+    @Override
     public int execScript(Map<String,?> props, String summaryForLogging, List<String> commands, Map<String,?> env) {
         return newExecWithLoggingHelpers().execScript(augmentPropertiesWithSshConfigGivenToProps(props), summaryForLogging, commands, env);
     }
@@ -866,15 +870,16 @@ public class SshMachineLocation extends AbstractMachineLocation implements Machi
         LOG.debug("installing {} to {} on {}, attempting remote curl", new Object[] { url, destPath, this });
 
         try {
+            BashCommandsConfigurable bash = BrooklynOsCommands.bash(this);
             PipedInputStream insO = new PipedInputStream(); OutputStream outO = new PipedOutputStream(insO);
             PipedInputStream insE = new PipedInputStream(); OutputStream outE = new PipedOutputStream(insE);
             StreamGobbler sgsO = new StreamGobbler(insO, null, LOG); sgsO.setLogPrefix("[curl @ "+address+":stdout] ").start();
-            StreamGobbler sgsE = new StreamGobbler(insE, null, LOG); sgsE.setLogPrefix("[curl @ "+address+":stdout] ").start();
+            StreamGobbler sgsE = new StreamGobbler(insE, null, LOG); sgsE.setLogPrefix("[curl @ "+address+":stderr] ").start();
             Map<String, ?> sshProps = MutableMap.<String, Object>builder().putAll(props).put("out", outO).put("err", outE).build();
             int result = execScript(sshProps, "copying remote resource "+url+" to server",  ImmutableList.of(
-                    BashCommands.INSTALL_CURL, // TODO should hold the 'installing' mutex
+                    bash.INSTALL_CURL, // TODO should hold the 'installing' mutex
                     "mkdir -p `dirname '"+destPath+"'`",
-                    "curl "+url+" -L --silent --insecure --show-error --fail --connect-timeout 60 --max-time 600 --retry 5 -o '"+destPath+"'"));
+                    "curl "+url+" -L --silent"+(bash.isIgnoreCerts() ? " --insecure" : "")+" --show-error --fail --connect-timeout 60 --max-time 600 --retry 5 -o '"+destPath+"'"));
             sgsO.close();
             sgsE.close();
             if (result != 0) {
@@ -901,7 +906,8 @@ public class SshMachineLocation extends AbstractMachineLocation implements Machi
 
     @Override
     public String toString() {
-        return "SshMachineLocation["+getDisplayName()+":"+user+"@"+address+":"+getPort()+"(id="+getId()+")]";
+        // previously we did getPort but that might not work eg if pointing at entity which is not managed
+        return "SshMachineLocation["+getDisplayName()+":"+user+"@"+address+"(id="+getId()+")]";
     }
 
     @Override
@@ -947,12 +953,22 @@ public class SshMachineLocation extends AbstractMachineLocation implements Machi
         }
     }
 
+    Duration sshCheckTimeout = null;
+    @Beta
+    public void setSshCheckTimeout(Duration sshCheckTimeout) {
+        this.sshCheckTimeout = sshCheckTimeout;
+    }
+    @Beta
+    public Duration getSshCheckTimeout() {
+        return Maybe.ofDisallowingNull(sshCheckTimeout).or(Duration.millis(SSHABLE_CONNECT_TIMEOUT));
+    }
+
     public boolean isSshable() {
         String cmd = "date";
         try {
             try {
                 Socket s = new Socket();
-                s.connect(new InetSocketAddress(getAddress(), getPort()), SSHABLE_CONNECT_TIMEOUT);
+                s.connect(new InetSocketAddress(getAddress(), getPort()), (int) getSshCheckTimeout().toMilliseconds());
                 s.close();
             } catch (IOException e) {
                 if (LOG.isDebugEnabled()) LOG.debug(""+this+" not [yet] reachable (socket "+getAddress()+":"+getPort()+"): "+e);
@@ -1002,7 +1018,7 @@ public class SshMachineLocation extends AbstractMachineLocation implements Machi
 
     /** returns the un-passphrased key-pair info if a key is being used, or else null */
     public KeyPair findKeyPair() {
-        OsCredential creds = LocationConfigUtils.getOsCredential(config().getBag());
+        OsCredential creds = LocationConfigUtils.getOsCredential(config().getBag(), ResourceUtils.create(this));
         if (creds.hasKey()) {
             String data = creds.getPrivateKeyData();
             return SecureKeys.readPem(data.getBytes(), getConfig(SshTool.PROP_PRIVATE_KEY_PASSPHRASE));
@@ -1047,10 +1063,10 @@ public class SshMachineLocation extends AbstractMachineLocation implements Machi
     @Override
     public String resolveOnBoxDirFor(Entity entity, String unresolvedPath) {
         ProcessTaskWrapper<Integer> baseTask = SshEffectorTasks.ssh(
-            BashCommands.alternatives("mkdir -p \"${BASE_DIR}\"",
-                BashCommands.chain(
-                    BashCommands.sudo("mkdir -p \"${BASE_DIR}\""),
-                    BashCommands.sudo("chown "+getUser()+" \"${BASE_DIR}\""))),
+            BrooklynOsCommands.bash(this).alternatives("mkdir -p \"${BASE_DIR}\"",
+                BrooklynOsCommands.bash(this).chain(
+                    BrooklynOsCommands.bash(this).sudo("mkdir -p \"${BASE_DIR}\""),
+                    BrooklynOsCommands.bash(this).sudo("chown "+getUser()+" \"${BASE_DIR}\""))),
             "cd ~",
             "cd ${BASE_DIR}",
             "echo BASE_DIR_RESULT':'`pwd`:BASE_DIR_RESULT")
