@@ -19,15 +19,16 @@
 
 package org.apache.brooklyn.core.config;
 
-import static org.testng.Assert.assertFalse;
-
-import java.util.Map;
-import java.util.concurrent.Callable;
-
+import com.google.common.base.Predicate;
+import com.google.common.base.Predicates;
+import com.google.common.collect.ImmutableList;
+import com.google.common.collect.ImmutableMap;
+import com.google.common.collect.Range;
 import org.apache.brooklyn.api.entity.Entity;
 import org.apache.brooklyn.api.entity.EntitySpec;
 import org.apache.brooklyn.api.entity.ImplementedBy;
 import org.apache.brooklyn.api.location.Location;
+import org.apache.brooklyn.api.mgmt.EntityManager;
 import org.apache.brooklyn.api.mgmt.Task;
 import org.apache.brooklyn.api.objs.BrooklynObject;
 import org.apache.brooklyn.api.policy.Policy;
@@ -44,9 +45,11 @@ import org.apache.brooklyn.core.test.entity.TestEntity;
 import org.apache.brooklyn.core.test.entity.TestEntityImpl;
 import org.apache.brooklyn.core.test.policy.TestPolicy;
 import org.apache.brooklyn.test.Asserts;
+import org.apache.brooklyn.util.collections.MutableList;
 import org.apache.brooklyn.util.collections.MutableMap;
 import org.apache.brooklyn.util.core.task.Tasks;
 import org.apache.brooklyn.util.javalang.JavaClassNames;
+import org.apache.brooklyn.util.math.MathPredicates;
 import org.apache.brooklyn.util.net.Networking;
 import org.apache.brooklyn.util.time.Duration;
 import org.apache.brooklyn.util.time.Time;
@@ -55,11 +58,13 @@ import org.slf4j.LoggerFactory;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
-import com.google.common.base.Predicate;
-import com.google.common.base.Predicates;
-import com.google.common.collect.ImmutableList;
-import com.google.common.collect.ImmutableMap;
-import com.google.common.collect.Range;
+import javax.annotation.Nonnull;
+import java.util.List;
+import java.util.Map;
+import java.util.concurrent.Callable;
+import java.util.function.Consumer;
+
+import static org.testng.Assert.assertFalse;
 
 public class ConfigKeyConstraintTest extends BrooklynAppUnitTestSupport {
 
@@ -67,6 +72,28 @@ public class ConfigKeyConstraintTest extends BrooklynAppUnitTestSupport {
     
     // ----------- Setup -----------------------------------------------------------------------------------------------
 
+    @ImplementedBy(EntityWithNonRequiredConstraintImpl.class)
+    public static interface EntityWithNonRequiredConstraint extends TestEntity {
+        ConfigKey<Number> OPTIONAL_CONFIG = ConfigKeys.builder(Number.class)
+                .name("test.conf.non-required.without-default")
+                .description("Configuration key that is optional")
+                .constraint(MathPredicates.greaterThan(2))
+                .build();
+    }
+    public static class EntityWithNonRequiredConstraintImpl extends TestEntityImpl implements EntityWithNonRequiredConstraint {
+    }
+
+    @ImplementedBy(EntityWithRequiredConstraintImpl.class)
+    public static interface EntityWithRequiredConstraint extends TestEntity {
+        ConfigKey<Object> REQUIRED_CONFIG = ConfigKeys.builder(Object.class)
+                .name("test.conf.required.without-default")
+                .description("Configuration key that is required")
+                .constraint(ConfigConstraints.required())
+                .build();
+    }
+    public static class EntityWithRequiredConstraintImpl extends TestEntityImpl implements EntityWithRequiredConstraint {
+    }
+
     @ImplementedBy(EntityWithNonNullConstraintImpl.class)
     public static interface EntityWithNonNullConstraint extends TestEntity {
         ConfigKey<Object> NON_NULL_CONFIG = ConfigKeys.builder(Object.class)
@@ -150,6 +177,16 @@ public class ConfigKeyConstraintTest extends BrooklynAppUnitTestSupport {
 
     // ----------- Tests -----------------------------------------------------------------------------------------------
 
+    @Test
+    public void testExceptionWhenEntityRequiredConfig() {
+        try {
+            app.createAndManageChild(EntitySpec.create(EntityWithRequiredConstraint.class));
+            Asserts.shouldHaveFailedPreviously("Expected exception when managing entity with missing config");
+        } catch (Exception e) {
+            Asserts.expectedFailureOfType(e, ConstraintViolationException.class);
+        }
+    }
+
     @Test
     public void testExceptionWhenEntityHasNullConfig() {
         try {
@@ -164,6 +201,13 @@ public class ConfigKeyConstraintTest extends BrooklynAppUnitTestSupport {
     public void testNoExceptionWhenEntityHasValueForRequiredConfig() {
         app.createAndManageChild(EntitySpec.create(EntityWithNonNullConstraint.class)
                 .configure(EntityWithNonNullConstraint.NON_NULL_CONFIG, new Object()));
+        app.createAndManageChild(EntitySpec.create(EntityWithRequiredConstraint.class)
+                .configure(EntityWithRequiredConstraint.REQUIRED_CONFIG, new Object()));
+    }
+
+    @Test
+    public void testNoExceptionWhenEntityHasNoValueForOptionalConfig() {
+        app.createAndManageChild(EntitySpec.create(EntityWithNonRequiredConstraint.class));
     }
 
     @Test
@@ -217,12 +261,11 @@ public class ConfigKeyConstraintTest extends BrooklynAppUnitTestSupport {
     }
 
     // Test fails because config keys that are not on an object's interfaces cannot be checked automatically.
-    @Test(enabled = false)
+    @Test
     public void testExceptionWhenPolicyHasNullForeignConfig() {
-        Policy p = mgmt.getEntityManager().createPolicy(PolicySpec.create(TestPolicy.class)
-                .configure(EntityWithNonNullConstraint.NON_NULL_CONFIG, (Object) null));
         try {
-            ConfigConstraints.assertValid(p);
+            Policy p = mgmt.getEntityManager().createPolicy(PolicySpec.create(TestPolicy.class)
+                    .configure(EntityWithNonNullConstraint.NON_NULL_CONFIG, (Object) null));
             Asserts.shouldHaveFailedPreviously("Expected exception when validating policy with missing config");
         } catch (Exception e) {
             Asserts.expectedFailureOfType(e, ConstraintViolationException.class);
@@ -463,4 +506,32 @@ public class ConfigKeyConstraintTest extends BrooklynAppUnitTestSupport {
             }
         }
     }
+
+    @Test
+    public void testCatchViolations() {
+        final EntitySpec<EntityWithNonNullConstraint> spec = EntitySpec.create(EntityWithNonNullConstraint.class);
+        spec.parent(app);
+
+        List<ConstraintViolationException> failedConstraints = MutableList.of();
+
+        EntityWithNonNullConstraint entity = mgmt().getEntityManager().createEntity(spec, new EntityManager.EntityCreationOptions() {
+            @Override
+            public <T extends Throwable> void onException(T e, @Nonnull Consumer<? super T> suggestedHandler) {
+                if (e instanceof ConstraintViolationException) failedConstraints.add( (ConstraintViolationException)e );
+                else suggestedHandler.accept(e);
+            }
+        });
+
+        Asserts.assertFalse(failedConstraints.isEmpty());
+        Asserts.assertStringContainsIgnoreCase(failedConstraints.toString(), entity.getDisplayName(), EntityWithNonNullConstraint.NON_NULL_CONFIG.getName(), "null");
+
+        // validation not subsequently done on access
+        entity.getConfig(EntityWithNonNullConstraint.NON_NULL_CONFIG);
+
+        // though it could be:
+//        Asserts.assertFailsWith(() -> entity.getConfig(EntityWithNonNullConstraint.NON_NULL_CONFIG),
+//                e -> Asserts.expectedFailureContainsIgnoreCase(e, "null") );
+    }
+
+
 }
