@@ -18,6 +18,9 @@
  */
 package org.apache.brooklyn.core.mgmt.rebind;
 
+import java.util.function.Supplier;
+import org.apache.brooklyn.core.entity.EntitySpecTest.TestEntityWithDefaultNameImpl;
+import org.apache.brooklyn.util.collections.MutableList;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertFalse;
 import static org.testng.Assert.assertNotNull;
@@ -29,6 +32,7 @@ import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.Semaphore;
@@ -52,6 +56,7 @@ import org.apache.brooklyn.api.sensor.SensorEvent;
 import org.apache.brooklyn.api.sensor.SensorEventListener;
 import org.apache.brooklyn.config.ConfigKey;
 import org.apache.brooklyn.core.config.BasicConfigKey;
+import org.apache.brooklyn.core.config.ConfigKeys;
 import org.apache.brooklyn.core.entity.AbstractEntity;
 import org.apache.brooklyn.core.entity.Entities;
 import org.apache.brooklyn.core.entity.EntityAsserts;
@@ -81,6 +86,7 @@ import org.testng.Assert;
 import org.testng.annotations.Test;
 
 import com.google.common.base.Objects;
+import com.google.common.base.Optional;
 import com.google.common.base.Predicates;
 import com.google.common.base.Suppliers;
 import com.google.common.base.Throwables;
@@ -285,7 +291,7 @@ public class RebindEntityTest extends RebindTestFixtureWithApp {
     // Where the same object is referenced from two different fields, using types that do not share a 
     // super type... then the object will just be deserialized once - at that point it must have *both*
     // interfaces.
-    @Test(groups="WIP")
+    @Test
     public void testHandlesReferencingOtherEntityInPojoFieldsOfOtherTypes() throws Exception {
         MyEntityWithMultipleInterfaces origE = origApp.createAndManageChild(EntitySpec.create(MyEntityWithMultipleInterfaces.class));
         ReffingEntity reffer = new ReffingEntity();
@@ -648,6 +654,7 @@ public class RebindEntityTest extends RebindTestFixtureWithApp {
 
         RebindTestUtils.waitForPersisted(origManagementContext);
         newManagementContext = RebindTestUtils.newPersistingManagementContextUnstarted(mementoDir, classLoader);
+
         List<Application> newApps = newManagementContext.getRebindManager().rebind(classLoader, null, ManagementNodeState.MASTER);
         newManagementContext.getRebindManager().startPersistence();
         
@@ -660,6 +667,72 @@ public class RebindEntityTest extends RebindTestFixtureWithApp {
         testRebindWhenPreviousAppDestroyedHasNoApp();
     }
 
+    @Test
+    public void testRebindAnonymousKeyDowncastedGivesCorrectType() throws Exception {
+        // happens if we write yaml and put an int where a double is expected
+        final String doubleKeyName = "double.key";
+        final ConfigKey<Object> keyAsObject = ConfigKeys.newConfigKey(Object.class, doubleKeyName);
+        final ConfigKey<Double> keyAsDouble = ConfigKeys.newDoubleConfigKey(doubleKeyName);
+        // set an int
+        origApp.config().set(keyAsObject, (int) 1);
+        // get the double when queried
+        Asserts.assertInstanceOf(origApp.config().get(keyAsDouble), Double.class);
+        // but doesn't actually know it's a double
+        Asserts.assertInstanceOf(origApp.config().get(keyAsObject), Integer.class);
+        // also assert the key isn't included in declared list
+        Optional<ConfigKey<?>> declaredKey = Iterables.tryFind(getTypeDeclaredKeys(origApp), (k) -> k.getName().equals(doubleKeyName));
+        if (declaredKey.isPresent()) Assert.fail("Shouldn't have declared anonymous key, but had: "+declaredKey.get());
+        
+        newApp = rebind();
+        // now (2017-11) this works because we check both types on lookup
+        Asserts.assertInstanceOf(newApp.config().get(keyAsDouble), Double.class);
+        // if not querying double, we get the original type
+        Asserts.assertInstanceOf(newApp.config().get(keyAsObject), Integer.class);
+        
+        // and this also succeeds because because now the anonymous key definition is not persisted
+        // (test changed, but confirmed it fails without the new BasicEntityMemento.isAnonymous check)
+        Optional<ConfigKey<?>> persistedKey = Iterables.tryFind(getTypeDeclaredKeys(newApp), (k) -> k.getName().equals(doubleKeyName));
+        if (persistedKey.isPresent()) Assert.fail("Shouldn't have persisted anonymous key, but had: "+persistedKey.get());
+    }
+    
+    @SuppressWarnings("unchecked")
+    @Test
+    public void testRebindNonAnonymousKeyAlsoGivesCorrectType() throws Exception {
+        // happens if we write yaml and put an int where a double is expected
+        final String doubleKeyName = "double.key";
+        final ConfigKey<Object> keyAsObject = ConfigKeys.newConfigKey(Object.class, doubleKeyName);
+        final ConfigKey<Double> keyAsDouble = ConfigKeys.newDoubleConfigKey(doubleKeyName);
+        // set an int, casting to allow it, but should be recording doubleness internally
+        origApp.config().set((ConfigKey<Object>) (ConfigKey<?>) keyAsDouble, (int) 1);
+        // get the double when queried
+        Asserts.assertInstanceOf(origApp.config().get(keyAsObject), Double.class);
+        // also assert the key isn't included in declared list
+        Optional<ConfigKey<?>> declaredKey = Iterables.tryFind(getTypeDeclaredKeys(origApp), (k) -> k.getName().equals(doubleKeyName));
+        if (declaredKey.isPresent()) Assert.fail("Shouldn't have declared anonymous key, but had: "+declaredKey.get());
+        
+        newApp = rebind();
+        // should know internally it's a double
+        Asserts.assertInstanceOf(newApp.config().get(keyAsDouble), Double.class);
+        Asserts.assertInstanceOf(newApp.config().get(keyAsObject), Double.class);
+        
+        // this works because we add the key to the type-declared items on persistence;
+        // this has always been the case, even though it introduces an asymmetry,
+        // it's the only place we currently have on the persisted entity to store keys
+        // (whereas the in-memory can keep dynamic keys in its map, not on the type;
+        // ideally the in-memory map would just use strings, and type would be added
+        // on set to the dynamic type if it's non-anonymous; see further notes in BasicEntityMemento.isAnonymous)
+        Optional<ConfigKey<?>> persistedKey = Iterables.tryFind(getTypeDeclaredKeys(newApp), (k) -> k.getName().equals(doubleKeyName));
+        if (!persistedKey.isPresent()) Assert.fail("Should have persisted non-anonymous key: "+persistedKey.get());
+    }
+
+    protected static Set<ConfigKey<?>> getTypeDeclaredKeys(Entity e) {
+        // thought this would work, but see comment on findKeysDeclared - it also includes present ones
+        //return e.config().findKeysDeclared(Predicates.alwaysTrue());
+        
+        // this is the right way:
+        return e.getEntityType().getConfigKeys();
+    }
+    
     /**
      * @deprecated since 0.7; support for rebinding old-style entities is deprecated
      */
@@ -687,6 +760,62 @@ public class RebindEntityTest extends RebindTestFixtureWithApp {
         assertFalse(newE.isRebinding());
     }
     
+    @Test
+    public void testDoNotRepersistOnTransientAttributeChanged() throws Exception {
+        final AttributeSensor<String> MY_TRANSIENT_ATTRIBUTE = Sensors.builder(String.class, "myTransientAttribute")
+                .persistence(SensorPersistenceMode.NONE)
+                .build();
+        
+        RebindTestUtils.waitForPersisted(mgmt());
+        assertFalse(RebindTestUtils.hasPendingPersists(mgmt()));
+
+        origApp.sensors().set(MY_TRANSIENT_ATTRIBUTE, "myval");
+        assertFalse(RebindTestUtils.hasPendingPersists(mgmt()));
+    }
+    
+    @Test
+    public void testDoNotRepersistOnSetAttributeWithSameValue() throws Exception {
+        final AttributeSensor<String> MY_ATTRIBUTE = Sensors.builder(String.class, "myAttribute").build();
+        
+        origApp.sensors().set(MY_ATTRIBUTE, "myval");
+        RebindTestUtils.waitForPersisted(mgmt());
+        
+        origApp.sensors().set(MY_ATTRIBUTE, "myval");
+        assertFalse(RebindTestUtils.hasPendingPersists(mgmt()));
+    }
+
+    @Test
+    public void testPojoFieldNotRebinded() throws Exception {
+        MyEntity2 my2 = origApp.createAndManageChild(EntitySpec.create(MyEntity2.class));
+        ((MyEntity2Impl) Entities.deproxy(my2)).obj = "hello";
+
+        ((MyEntity2Impl) Entities.deproxy(my2)).requestPersist();
+        //mgmt().getRebindManager().forcePersistNow(true, null);
+
+        TestApplication app2 = rebind();
+        MyEntity2 my2b = (MyEntity2) app2.getChildren().iterator().next();
+
+        // fields not persisted
+        // Asserts.assertEquals( ((MyEntity2Impl) Entities.deproxy(my2b)).obj, "hello");
+        Asserts.assertNull( ((MyEntity2Impl) Entities.deproxy(my2b)).obj);
+    }
+
+    @Test
+    public void testCreationWithLambdaFails() throws Exception {
+        Collection<Entity> oldChildren = MutableList.copyOf( origApp.getChildren() );
+        Collection<Entity> oldEntities = MutableList.copyOf( mgmt().getEntityManager().getEntities() );
+
+        Asserts.assertFailsWith(() -> origApp.createAndManageChild(EntitySpec.create(TestEntity.class)
+                        .configure(TestEntityWithDefaultNameImpl.CONF_OBJECT, (Supplier<String>) () -> "hello")),
+                error -> {
+                    Asserts.expectedFailureContainsIgnoreCase(error, "lambda");
+                    return true;
+                });
+
+        Asserts.assertEquals( origApp.getChildren(), oldChildren );
+        Asserts.assertEquals( mgmt().getEntityManager().getEntities(), oldEntities );
+    }
+
     @ImplementedBy(EntityChecksIsRebindingImpl.class)
     public static interface EntityChecksIsRebinding extends TestEntity {
         boolean isRebindingValWhenRebinding();
@@ -731,7 +860,7 @@ public class RebindEntityTest extends RebindTestFixtureWithApp {
     
     public static class MyEntityImpl extends AbstractEntity implements MyEntity {
         @SuppressWarnings("unused")
-        private final Object dummy = new Object(); // so not serializable
+        private final Object dummy = new Object(); // not a problem that this is not serializable (not written anyway)
 
         public MyEntityImpl() {
         }
@@ -761,9 +890,6 @@ public class RebindEntityTest extends RebindTestFixtureWithApp {
     }
     
     public static class MyEntityWithMultipleInterfacesImpl extends AbstractGroupImpl implements MyEntityWithMultipleInterfaces {
-        @SuppressWarnings("unused")
-        private final Object dummy = new Object(); // so not serializable
-
         public MyEntityWithMultipleInterfacesImpl() {
         }
 
@@ -806,9 +932,6 @@ public class RebindEntityTest extends RebindTestFixtureWithApp {
         public static final AttributeSensor<Location> LOCATION_REF_SENSOR = new BasicAttributeSensor<Location>(
                 Location.class, "test.attribute.locationref", "Ref to other location");
         
-        @SuppressWarnings("unused")
-        private final Object dummy = new Object(); // so not serializable
-
         public MyEntityReffingOthersImpl() {
         }
     }
@@ -829,8 +952,8 @@ public class RebindEntityTest extends RebindTestFixtureWithApp {
     public static class MyEntity2Impl extends AbstractEntity implements MyEntity2 {
         final List<String> events = new CopyOnWriteArrayList<String>();
 
-        @SuppressWarnings("unused")
-        private final Object dummy = new Object(); // so not serializable
+        @SetFromFlag
+        Object obj;
 
         public MyEntity2Impl() {
         }
