@@ -136,6 +136,7 @@ public class RebindFeedTest extends RebindTestFixtureWithApp {
     }
 
     @Test(groups="Integration", invocationCount=50)
+    // TODO occasionally this fails at the start, the first 'EntityAsserts.assertAttributeEqualsEventually(origEntity, SENSOR_INT, 200);' is never satisfied, it just shows 'null'
     public void testHttpFeedRegisteredInInitIsPersistedAndFeedsStopManyTimes() throws Exception {
         testHttpFeedRegisteredInInitIsPersistedAndFeedsStop();
     }
@@ -295,18 +296,18 @@ public class RebindFeedTest extends RebindTestFixtureWithApp {
         @Override
         public void init() {
             super.init();
-            addFeed(FunctionFeed.builder()
+            FunctionFeed.builder()
                     .entity(this)
                     .poll(FunctionPollConfig.forSensor(SENSOR_INT)
                             .period(POLL_PERIOD)
                             .callable(Callables.returning(1)))
-                    .build());
+                    .build(true);
             addFeed(FunctionFeed.builder()
                     .entity(this)
                     .poll(FunctionPollConfig.forSensor(SENSOR_STRING)
                             .period(POLL_PERIOD)
                             .callable(Callables.returning("OK")))
-                    .build());
+                    .build());  // should be identical to above build(true)
         }
     }
     
