@@ -18,15 +18,18 @@
  */
 package org.apache.brooklyn.launcher.common;
 
-import static com.google.common.base.Preconditions.checkNotNull;
-
 import java.io.File;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
-
 import javax.annotation.Nullable;
 
+import com.google.common.annotations.Beta;
+import com.google.common.annotations.VisibleForTesting;
+import com.google.common.base.Splitter;
+import com.google.common.collect.ImmutableList;
+import com.google.common.collect.Lists;
+import com.google.common.collect.Maps;
 import org.apache.brooklyn.api.entity.Application;
 import org.apache.brooklyn.api.entity.EntitySpec;
 import org.apache.brooklyn.api.location.Location;
@@ -44,7 +47,7 @@ import org.apache.brooklyn.camp.brooklyn.BrooklynCampPlatformLauncherNoServer;
 import org.apache.brooklyn.config.ConfigKey;
 import org.apache.brooklyn.core.catalog.internal.CatalogInitialization;
 import org.apache.brooklyn.core.entity.Attributes;
-import org.apache.brooklyn.core.entity.Entities;
+import org.apache.brooklyn.core.entity.Dumper;
 import org.apache.brooklyn.core.entity.trait.Startable;
 import org.apache.brooklyn.core.internal.BrooklynProperties;
 import org.apache.brooklyn.core.mgmt.EntityManagementUtils;
@@ -75,11 +78,7 @@ import org.apache.brooklyn.util.time.Duration;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import com.google.common.annotations.Beta;
-import com.google.common.base.Splitter;
-import com.google.common.collect.ImmutableList;
-import com.google.common.collect.Lists;
-import com.google.common.collect.Maps;
+import static com.google.common.base.Preconditions.checkNotNull;
 
 /**
  * Example usage is:
@@ -90,7 +89,7 @@ import com.google.common.collect.Maps;
  *     .location("localhost")
  *     .start();
  * 
- * Entities.dumpInfo(launcher.getApplications());
+ * Dumper.dumpInfo(launcher.getApplications());
  * </pre>
  */
 public class BasicLauncher<T extends BasicLauncher<T>> {
@@ -158,7 +157,7 @@ public class BasicLauncher<T extends BasicLauncher<T>> {
      * The application will not be started as part of this call (callers can
      * subsequently call {@link #start()} or {@link #getApplications()}.
      *
-     * @see #application(Application)
+     * @see #application(org.apache.brooklyn.api.entity.EntitySpec)
      */
     public T application(String yaml) {
         this.yamlAppsToManage.add(yaml);
@@ -253,12 +252,12 @@ public class BasicLauncher<T extends BasicLauncher<T>> {
     }
 
     public T persistMode(PersistMode persistMode) {
-        this.persistMode = persistMode;
+        this.persistMode = checkNotNull(persistMode, "persistMode");
         return self();
     }
 
     public T highAvailabilityMode(HighAvailabilityMode highAvailabilityMode) {
-        this.highAvailabilityMode = highAvailabilityMode;
+        this.highAvailabilityMode = checkNotNull(highAvailabilityMode, "highAvailabilityMode");
         return self();
     }
 
@@ -333,7 +332,7 @@ public class BasicLauncher<T extends BasicLauncher<T>> {
             LOG.info("Copying persisted state to "+destinationDir+(destinationLocationSpec!=null ? " @ "+destinationLocationSpec : ""));
             PersistenceObjectStore destinationObjectStore = BrooklynPersistenceUtils.newPersistenceObjectStore(
                 managementContext, destinationLocationSpec, destinationDir);
-            BrooklynPersistenceUtils.writeMemento(managementContext, memento, destinationObjectStore);
+            BrooklynPersistenceUtils.writeMemento(managementContext, memento, destinationObjectStore, "copy on launch");
             BrooklynPersistenceUtils.writeManagerMemento(managementContext, planeState, destinationObjectStore);
         } catch (Exception e) {
             Exceptions.propagateIfFatal(e);
@@ -364,11 +363,12 @@ public class BasicLauncher<T extends BasicLauncher<T>> {
      */
     // Make private after deprecation
     @Deprecated
+    @VisibleForTesting
     public void persistState(BrooklynMementoRawData memento, String destinationDir, @Nullable String destinationLocationSpec) {
         initManagementContext();
         PersistenceObjectStore destinationObjectStore = BrooklynPersistenceUtils.newPersistenceObjectStore(
             managementContext, destinationLocationSpec, destinationDir);
-        BrooklynPersistenceUtils.writeMemento(managementContext, memento, destinationObjectStore);
+        BrooklynPersistenceUtils.writeMemento(managementContext, memento, destinationObjectStore, "persist on launch (for testing only)");
     }
 
     /**
@@ -389,6 +389,8 @@ public class BasicLauncher<T extends BasicLauncher<T>> {
         if (started) throw new IllegalStateException("Cannot start() or launch() multiple times");
         started = true;
 
+        checkConfiguration();
+        
         initManagementContext();
 
         CatalogInitialization catInit = ((ManagementContextInternal)managementContext).getCatalogInitialization();
@@ -407,6 +409,23 @@ public class BasicLauncher<T extends BasicLauncher<T>> {
         return self();
     }
 
+    private void checkConfiguration() {
+        if (highAvailabilityMode != HighAvailabilityMode.DISABLED) {
+            if (persistMode == PersistMode.DISABLED) {
+                if (highAvailabilityMode == HighAvailabilityMode.AUTO) {
+                    highAvailabilityMode = HighAvailabilityMode.DISABLED;
+                } else {
+                    throw new FatalConfigurationRuntimeException("Cannot specify highAvailability when persistence is disabled");
+                }
+            } else if (persistMode == PersistMode.CLEAN && 
+                    (highAvailabilityMode == HighAvailabilityMode.STANDBY 
+                            || highAvailabilityMode == HighAvailabilityMode.HOT_STANDBY
+                            || highAvailabilityMode == HighAvailabilityMode.HOT_BACKUP)) {
+                throw new FatalConfigurationRuntimeException("Cannot specify highAvailability "+highAvailabilityMode+" when persistence is CLEAN");
+            }
+        }
+    }
+
     /**
      * Starts the web server (with web console) and Brooklyn applications, as per the specifications configured. 
      * @return An object containing details of the web server and the management context.
@@ -415,8 +434,13 @@ public class BasicLauncher<T extends BasicLauncher<T>> {
         if (startedPartTwo) throw new IllegalStateException("Cannot start() or launch() multiple times");
         startedPartTwo = true;
         
-        handlePersistence();
-        populateCatalog(catalogInitialization);
+        if (persistMode != PersistMode.DISABLED) {
+            handlePersistence();
+        } else {
+            ((LocalManagementContext)managementContext).generateManagementPlaneId();
+            populateInitialCatalogNoPersistence(catalogInitialization);
+            managementContext.getHighAvailabilityManager().disabled(false);
+        }
         markCatalogStarted(catalogInitialization);
         addLocations();
         markStartupComplete();
@@ -471,34 +495,33 @@ public class BasicLauncher<T extends BasicLauncher<T>> {
     protected void startingUp() {
     }
 
-    private void markCatalogStarted(CatalogInitialization catInit) {
-        catInit.setStartingUp(false);
-    }
-
-    protected void populateCatalog(CatalogInitialization catInit) {
+    protected void populateInitialCatalogNoPersistence(CatalogInitialization catInit) {
+        assert persistMode == PersistMode.DISABLED : "persistMode="+persistMode;
+        
         try {
-            // run cat init now if it hasn't yet been run; 
-            // will also run if there was an ignored error in catalog above, allowing it to fail startup here if requested
-            if (catInit!=null && !catInit.hasRunOfficialInitialization()) {
-                if (persistMode==PersistMode.DISABLED) {
-                    LOG.debug("Loading catalog as part of launch sequence (it was not loaded as part of any rebind sequence)");
-                    catInit.populateCatalog(ManagementNodeState.MASTER, true, true, null);
-                } else {
-                    // should have loaded during rebind
-                    ManagementNodeState state = managementContext.getHighAvailabilityManager().getNodeState();
-                    LOG.warn("Loading catalog for "+state+" as part of launch sequence (it was not loaded as part of the rebind sequence)");
-                    catInit.populateCatalog(state, true, true, null);
-                }
-            }
+            LOG.debug("Initializing initial catalog as part of launch sequence (prior to any persistence rebind)");
+            catInit.populateInitialCatalogOnly();
         } catch (Exception e) {
             handleSubsystemStartupError(ignoreCatalogErrors, "initial catalog", e);
         }
     }
 
+    private void markCatalogStarted(CatalogInitialization catInit) {
+        catInit.setStartingUp(false);
+    }
+
     protected void handlePersistence() {
         try {
-            initPersistence();
-            startPersistence();
+            if (managementContext.getRebindManager().getPersister()!=null) {
+                LOG.debug("Persister already initialized by mgmt context; skipping from launcher");
+                // call below will fail if persister already set eg by RebindTestUtils
+                if (!(((RebindManagerImpl) managementContext.getRebindManager()).isPersistenceRunning())) {
+                    throw new IllegalStateException("Persistence was initialized but not running, prior to launcher");
+                }
+            } else {
+                initPersistence();
+                startPersistence();
+            }
         } catch (Exception e) {
             handleSubsystemStartupError(ignorePersistenceErrors, "persistence", e);
         }
@@ -551,45 +574,38 @@ public class BasicLauncher<T extends BasicLauncher<T>> {
     }
 
     protected void initPersistence() {
-        // Prepare the rebind directory, and initialise the RebindManager as required
+        assert persistMode != PersistMode.DISABLED : "persistMode="+persistMode+"; haMode="+highAvailabilityMode;
+
+        // Prepare the rebind directory, and initialise the RebindManager
         final PersistenceObjectStore objectStore;
-        if (persistMode == PersistMode.DISABLED) {
-            LOG.info("Persistence disabled");
-            objectStore = null;
-            ((LocalManagementContext)managementContext).generateManagementPlaneId();
-        } else {
-            try {
-                if (persistenceLocation == null) {
-                    persistenceLocation = brooklynProperties.getConfig(BrooklynServerConfig.PERSISTENCE_LOCATION_SPEC);
-                }
-                persistenceDir = BrooklynServerPaths.newMainPersistencePathResolver(brooklynProperties).location(persistenceLocation).dir(persistenceDir).resolve();
-                objectStore = BrooklynPersistenceUtils.newPersistenceObjectStore(managementContext, persistenceLocation, persistenceDir, 
-                    persistMode, highAvailabilityMode);
-                    
-                RebindManager rebindManager = managementContext.getRebindManager();
-                
-                BrooklynMementoPersisterToObjectStore persister = new BrooklynMementoPersisterToObjectStore(
-                    objectStore, managementContext);
-                PersistenceExceptionHandler persistenceExceptionHandler = PersistenceExceptionHandlerImpl.builder().build();
-                ((RebindManagerImpl) rebindManager).setPeriodicPersistPeriod(persistPeriod);
-                rebindManager.setPersister(persister, persistenceExceptionHandler);
-            } catch (FatalConfigurationRuntimeException e) {
-                throw e;
-            } catch (Exception e) {
-                Exceptions.propagateIfFatal(e);
-                LOG.debug("Error initializing persistence subsystem (rethrowing): "+e, e);
-                throw new FatalRuntimeException("Error initializing persistence subsystem: "+
-                    Exceptions.collapseText(e), e);
+        try {
+            if (persistenceLocation == null) {
+                persistenceLocation = brooklynProperties.getConfig(BrooklynServerConfig.PERSISTENCE_LOCATION_SPEC);
             }
+            persistenceDir = BrooklynServerPaths.newMainPersistencePathResolver(brooklynProperties).location(persistenceLocation).dir(persistenceDir).resolve();
+            objectStore = BrooklynPersistenceUtils.newPersistenceObjectStore(managementContext, persistenceLocation, persistenceDir, 
+                persistMode, highAvailabilityMode);
+                
+            RebindManager rebindManager = managementContext.getRebindManager();
+            
+            BrooklynMementoPersisterToObjectStore persister = new BrooklynMementoPersisterToObjectStore(
+                objectStore, managementContext);
+            PersistenceExceptionHandler persistenceExceptionHandler = PersistenceExceptionHandlerImpl.builder().build();
+            ((RebindManagerImpl) rebindManager).setPeriodicPersistPeriod(persistPeriod);
+            rebindManager.setPersister(persister, persistenceExceptionHandler);
+        } catch (FatalConfigurationRuntimeException e) {
+            throw e;
+        } catch (Exception e) {
+            Exceptions.propagateIfFatal(e);
+            LOG.debug("Error initializing persistence subsystem (rethrowing): "+e, e);
+            throw new FatalRuntimeException("Error initializing persistence subsystem: "+
+                Exceptions.collapseText(e), e);
         }
         
         // Initialise the HA manager as required
         if (highAvailabilityMode == HighAvailabilityMode.DISABLED) {
             LOG.info("High availability disabled");
         } else {
-            if (objectStore==null)
-                throw new FatalConfigurationRuntimeException("Cannot run in HA mode when no persistence configured.");
-
             HighAvailabilityManager haManager = managementContext.getHighAvailabilityManager();
             ManagementPlaneSyncRecordPersister persister =
                 new ManagementPlaneSyncRecordPersisterToObjectStore(managementContext,
@@ -602,20 +618,20 @@ public class BasicLauncher<T extends BasicLauncher<T>> {
     }
     
     protected void startPersistence() {
+        assert persistMode != PersistMode.DISABLED : "persistMode="+persistMode+"; haMode="+highAvailabilityMode;
+        
         // Now start the HA Manager and the Rebind manager, as required
         if (highAvailabilityMode == HighAvailabilityMode.DISABLED) {
             HighAvailabilityManager haManager = managementContext.getHighAvailabilityManager();
-            haManager.disabled();
+            haManager.disabled(true);
 
-            if (persistMode != PersistMode.DISABLED) {
-                startPersistenceWithoutHA();
-            }
+            startPersistenceWithoutHA();
             
         } else {
             // Let the HA manager decide when objectstore.prepare and rebindmgr.rebind need to be called 
             // (based on whether other nodes in plane are already running).
             
-            HighAvailabilityMode startMode=null;
+            HighAvailabilityMode startMode;
             switch (highAvailabilityMode) {
                 case AUTO:
                 case MASTER:
@@ -626,9 +642,9 @@ public class BasicLauncher<T extends BasicLauncher<T>> {
                     break;
                 case DISABLED:
                     throw new IllegalStateException("Unexpected code-branch for high availability mode "+highAvailabilityMode);
+                default:
+                    throw new IllegalStateException("Unexpected high availability mode "+highAvailabilityMode);
             }
-            if (startMode==null)
-                throw new IllegalStateException("Unexpected high availability mode "+highAvailabilityMode);
             
             LOG.debug("Management node (with HA) starting");
             HighAvailabilityManager haManager = managementContext.getHighAvailabilityManager();
@@ -706,7 +722,7 @@ public class BasicLauncher<T extends BasicLauncher<T>> {
                 try {
                     LOG.info("Starting brooklyn application {} in location{} {}", new Object[] { app, locations.size()!=1?"s":"", locations });
                     ((Startable)app).start(locations);
-                    Entities.dumpInfo(app);
+                    Dumper.dumpInfo(app);
                     String sensors = "";
                     if (app.getAttribute(Attributes.MAIN_URI_MAPPED_PUBLIC)!=null) {
                         sensors = ": "+app.getAttribute(Attributes.MAIN_URI_MAPPED_PUBLIC);
