@@ -30,6 +30,7 @@ import java.util.Stack;
 import javax.annotation.Nonnull;
 
 import org.apache.brooklyn.util.guava.Maybe;
+import org.apache.brooklyn.util.javalang.Boxing;
 import org.apache.brooklyn.util.text.StringEscapes.JavaStringEscapes;
 
 import com.google.common.annotations.Beta;
@@ -109,6 +110,17 @@ public class Jsonya {
         if (x==null) return true;
         return convertToJsonPrimitive(x).equals(x);
     }
+    public static boolean isTypeJsonPrimitiveCompatible(Object x) {
+        if (x==null) return true;
+        return x instanceof Map || x instanceof Collection || x instanceof String || Boxing.isPrimitiveOrBoxedObject(x);
+    }
+    public static boolean isJsonPrimitiveDeep(Object x) {
+        if (x==null) return true;
+        if (x instanceof String || Boxing.isPrimitiveOrBoxedObject(x)) return true;
+        if (x instanceof Map) return !((Map<?,?>)x).entrySet().stream().anyMatch(ent -> !isJsonPrimitiveDeep(ent.getKey()) || !isJsonPrimitiveDeep(ent.getValue()));
+        if (x instanceof Collection) return !((Collection)x).stream().anyMatch(ent -> !isJsonPrimitiveDeep(ent));
+        return false;
+    }
 
     @SuppressWarnings({"rawtypes","unchecked"})
     public static class Navigator<T extends Map<?,?>> {
